@@ -30,7 +30,7 @@
 #define BESSEL_H
 
 #include <itpp/base/vec.h>
-
+#include <itpp/itexports.h>
 
 namespace itpp
 {
@@ -48,25 +48,25 @@ namespace itpp
   \f]
   where \f$\nu\f$ is the order and \f$ 0 < x < \infty \f$.
 */
-double besselj(int nu, double x);
+ITPP_EXPORT double besselj(int nu, double x);
 
 /*!
   \ingroup besselfunctions
   \brief Bessel function of first kind of order \a nu for \a nu integer
 */
-vec besselj(int nu, const vec &x);
+ITPP_EXPORT vec besselj(int nu, const vec &x);
 
 /*!
   \ingroup besselfunctions
   \brief Bessel function of first kind of order \a nu. \a nu is real.
 */
-double besselj(double nu, double x);
+ITPP_EXPORT double besselj(double nu, double x);
 
 /*!
   \ingroup besselfunctions
   \brief Bessel function of first kind of order \a nu. \a nu is real.
 */
-vec besselj(double nu, const vec &x);
+ITPP_EXPORT vec besselj(double nu, const vec &x);
 
 /*!
   \ingroup besselfunctions
@@ -78,25 +78,25 @@ vec besselj(double nu, const vec &x);
   \f]
   where \f$\nu\f$ is the order and \f$ 0 < x < \infty \f$.
 */
-double bessely(int nu, double x);
+ITPP_EXPORT double bessely(int nu, double x);
 
 /*!
   \ingroup besselfunctions
   \brief Bessel function of second kind of order \a nu. \a nu is integer.
 */
-vec bessely(int nu, const vec &x);
+ITPP_EXPORT vec bessely(int nu, const vec &x);
 
 /*!
   \ingroup besselfunctions
   \brief Bessel function of second kind of order \a nu. \a nu is real.
 */
-double bessely(double nu, double x);
+ITPP_EXPORT double bessely(double nu, double x);
 
 /*!
   \ingroup besselfunctions
   \brief Bessel function of second kind of order \a nu. \a nu is real.
 */
-vec bessely(double nu, const vec &x);
+ITPP_EXPORT vec bessely(double nu, const vec &x);
 
 /*!
   \ingroup besselfunctions
@@ -108,13 +108,13 @@ vec bessely(double nu, const vec &x);
   \f]
   where \f$\nu\f$ is the order and \f$ 0 < x < \infty \f$.
 */
-double besseli(double nu, double x);
+ITPP_EXPORT double besseli(double nu, double x);
 
 /*!
   \ingroup besselfunctions
   \brief Modified Bessel function of first kind of order \a nu. \a nu is \a double. \a x is \a double.
 */
-vec besseli(double nu, const vec &x);
+ITPP_EXPORT vec besseli(double nu, const vec &x);
 
 /*!
   \ingroup besselfunctions
@@ -126,13 +126,13 @@ vec besseli(double nu, const vec &x);
   \f]
   where \f$\nu\f$ is the order and \f$ 0 < x < \infty \f$.
 */
-double besselk(int nu, double x);
+ITPP_EXPORT double besselk(int nu, double x);
 
 /*!
   \ingroup besselfunctions
   \brief Modified Bessel function of second kind of order \a nu. \a nu is double. \a x is double.
 */
-vec besselk(int nu, const vec &x);
+ITPP_EXPORT vec besselk(int nu, const vec &x);
 
 } //namespace itpp
 
