@@ -5,24 +5,23 @@
  *
  * -------------------------------------------------------------------------
  *
- * IT++ - C++ library of mathematical, signal processing, speech processing,
- *        and communications classes and functions
+ * Copyright (C) 1995-2010  (see AUTHORS file for a list of contributors)
  *
- * Copyright (C) 1995-2009  (see AUTHORS file for a list of contributors)
+ * This file is part of IT++ - a C++ library of mathematical, signal
+ * processing, speech processing, and communications classes and functions.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * IT++ is free software: you can redistribute it and/or modify it under the
+ * terms of the GNU General Public License as published by the Free Software
+ * Foundation, either version 3 of the License, or (at your option) any
+ * later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * IT++ is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ * You should have received a copy of the GNU General Public License along
+ * with IT++.  If not, see <http://www.gnu.org/licenses/>.
  *
  * -------------------------------------------------------------------------
  */
@@ -134,7 +133,6 @@ bool it_ifile::read_check_file_header()
 
 void it_ifile::read_data_header(data_header &h)
 {
-  std::streampos p = s.tellg();
   s.clear();
   s >> h.hdr_bytes;
   s >> h.data_bytes;
@@ -503,11 +501,10 @@ void it_ifile::low_level_read_hi(Array<std::complex<double> > &v)
 // it_file class
 // ----------------------------------------------------------------------
 
-it_file::it_file(): low_prec(false), next_name(""), next_desc(""),
-    fname("") {}
+it_file::it_file(): low_prec(false), _strings(new Strings_Holder) {}
 
 it_file::it_file(const std::string &name, bool trunc):
-    low_prec(false), next_name(""), next_desc(""), fname("")
+    low_prec(false), _strings(new Strings_Holder)
 {
   open(name, trunc);
 }
@@ -527,7 +524,7 @@ void it_file::open(const std::string &name, bool trunc)
     it_error("it_file::open(): Corrupt file (not an it_file)");
   }
 
-  fname = name;
+  fname() = name;
 }
 
 void it_file::close()
@@ -548,11 +545,11 @@ void it_file::write_file_header()
 
 void it_file::write_data_header(const std::string &type, uint64_t size)
 {
-  it_error_if(next_name == "", "it_file::write_data_header(): Can not "
+  it_error_if(next_name() == "", "it_file::write_data_header(): Can not "
               "write without a name");
-  write_data_header(type, next_name, size, next_desc);
-  next_name = "";
-  next_desc = "";
+  write_data_header(type, next_name(), size, next_desc());
+  next_name() = "";
+  next_desc() = "";
 }
 
 void it_file::write_data_header(const std::string &type,
@@ -660,7 +657,7 @@ void it_file::pack()
   s.clear();
 
   // allocate buffer of size equal to file size
-  char* buffer = new char[p];
+  char* buffer = new char[int(p)];
   char* b_ptr = buffer;
 
   // copy file header and start counting the size of compacted file
@@ -688,7 +685,7 @@ void it_file::pack()
 
   // close and reopen file truncating it
   s.close();
-  s.open(fname, true, bfstream_base::l_endian);
+  s.open(fname(), true, bfstream_base::l_endian);
   // write compacted data to the reopend empty file
   for (uint64_t i = 0; i < size; ++i)
     s.put(buffer[i]);
@@ -1754,7 +1751,7 @@ it_file &operator<<(it_file &f, const Array<std::string> &v)
   // calculate total length of Array
   int sum_l = 0;
   for (int i = 0; i < v.size(); ++i)
-    sum_l += v(i).size();
+    sum_l += int(v(i).size());
 
   // write header
   f.write_data_header("stringArray", sizeof(uint64_t) * (1 + v.size())
@@ -2319,16 +2316,15 @@ void it_ifile_old::low_level_read_hi(Array<std::complex<double> > &v)
   }
 }
 
-it_file_old::it_file_old()
+it_file_old::it_file_old():_string(new String_Holder())
 {
   low_prec = false;
-  next_name = "";
 }
 
-it_file_old::it_file_old(const std::string &name, bool trunc)
+it_file_old::it_file_old(const std::string &name, bool trunc):
+_string(new String_Holder())
 {
   low_prec = false;
-  next_name = "";
   open(name, trunc);
 }
 
@@ -2366,9 +2362,9 @@ void it_file_old::write_file_header()
 
 void it_file_old::write_data_header(const std::string &type, uint32_t size)
 {
-  it_error_if(next_name == "", "Try to write without a name");
-  write_data_header(type, next_name, size);
-  next_name = "";
+  it_error_if(next_name() == "", "Try to write without a name");
+  write_data_header(type, next_name(), size);
+  next_name() = "";
 }
 
 void it_file_old::write_data_header(const std::string &type,
@@ -2381,7 +2377,7 @@ void it_file_old::write_data_header(const std::string &type,
   int skip;
 
   h1.endianity = static_cast<char>(s.get_native_endianity());
-  h1.hdr_bytes = 1 + 3 * 4 + type.size() + 1 + name.size() + 1;
+  h1.hdr_bytes = 1 + 3 * 4 + int(type.size()) + 1 + int(name.size()) + 1;
   h1.data_bytes = size;
   h1.block_bytes = h1.hdr_bytes + h1.data_bytes;
   h1.name = name;
@@ -2413,7 +2409,7 @@ void it_file_old::write_data_header(const std::string &type,
     if (availpos == 0) {
       if (h2.type == "" && h2.block_bytes >= h1.block_bytes) {
         h1.block_bytes = h2.block_bytes;
-        availpos = p;
+        availpos = int(p);
       }
       else if (h2.block_bytes - h2.hdr_bytes - h2.data_bytes >= h1.block_bytes) {
         h1.block_bytes = h2.block_bytes - h2.hdr_bytes - h2.data_bytes;
@@ -2566,7 +2562,7 @@ void it_file_old::low_level_write(const cvec &v)
 
 void it_file_old::low_level_write(const std::string &str)
 {
-  int size = str.size();
+  int size = int(str.size());
   s << static_cast<int32_t>(size);
 
   for (int i = 0; i < size; i++)
@@ -3292,7 +3288,7 @@ it_file_old &operator<<(it_file_old &f, const cvec &v)
 
 it_file_old &operator<<(it_file_old &f, const std::string &str)
 {
-  f.write_data_header("string", sizeof(int) + str.size() * sizeof(char));
+  f.write_data_header("string", sizeof(int) + int(str.size()) * sizeof(char));
   f.low_level_write(str);
 
   return f;
@@ -3480,7 +3476,7 @@ it_file_old &operator<<(it_file_old &f, const Array<std::string> &v)
 
   // calculate total length of Array
   for (i = 0; i < v.size(); i++) {
-    sum_l += v(i).size();
+    sum_l += int(v(i).size());
   }
 
   // write header
