@@ -6,24 +6,23 @@
  *
  * -------------------------------------------------------------------------
  *
- * IT++ - C++ library of mathematical, signal processing, speech processing,
- *        and communications classes and functions
+ * Copyright (C) 1995-2010  (see AUTHORS file for a list of contributors)
  *
- * Copyright (C) 1995-2008  (see AUTHORS file for a list of contributors)
+ * This file is part of IT++ - a C++ library of mathematical, signal
+ * processing, speech processing, and communications classes and functions.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * IT++ is free software: you can redistribute it and/or modify it under the
+ * terms of the GNU General Public License as published by the Free Software
+ * Foundation, either version 3 of the License, or (at your option) any
+ * later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * IT++ is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ * You should have received a copy of the GNU General Public License along
+ * with IT++.  If not, see <http://www.gnu.org/licenses/>.
  *
  * -------------------------------------------------------------------------
  */
@@ -33,7 +32,8 @@
 
 #include <itpp/base/vec.h>
 #include <itpp/base/mat.h>
-
+#include <itpp/base/converters.h>
+#include <itpp/itexports.h>
 
 namespace itpp
 {
@@ -42,7 +42,7 @@ namespace itpp
   \brief Return a integer vector with indicies where bvec == 1
   \ingroup miscfunc
 */
-ivec find(const bvec &invector);
+ITPP_EXPORT ivec find(const bvec &invector);
 
 /*!
   \addtogroup specmat
@@ -52,57 +52,85 @@ ivec find(const bvec &invector);
 //!@{
 
 //! A float vector of ones
-vec ones(int size);
+ITPP_EXPORT vec ones(int size);
 //! A Binary vector of ones
-bvec ones_b(int size);
+ITPP_EXPORT bvec ones_b(int size);
 //! A Int vector of ones
-ivec ones_i(int size);
+ITPP_EXPORT ivec ones_i(int size);
 //! A float Complex vector of ones
-cvec ones_c(int size);
+ITPP_EXPORT cvec ones_c(int size);
 
 //! A float (rows,cols)-matrix of ones
-mat ones(int rows, int cols);
+ITPP_EXPORT mat ones(int rows, int cols);
 //! A Binary (rows,cols)-matrix of ones
-bmat ones_b(int rows, int cols);
+ITPP_EXPORT bmat ones_b(int rows, int cols);
 //! A Int (rows,cols)-matrix of ones
-imat ones_i(int rows, int cols);
+ITPP_EXPORT imat ones_i(int rows, int cols);
 //! A Double Complex (rows,cols)-matrix of ones
-cmat ones_c(int rows, int cols);
+ITPP_EXPORT cmat ones_c(int rows, int cols);
 
 //! A Double vector of zeros
-vec zeros(int size);
+ITPP_EXPORT vec zeros(int size);
 //! A Binary vector of zeros
-bvec zeros_b(int size);
+ITPP_EXPORT bvec zeros_b(int size);
 //! A Int vector of zeros
-ivec zeros_i(int size);
+ITPP_EXPORT ivec zeros_i(int size);
 //! A Double Complex vector of zeros
-cvec zeros_c(int size);
+ITPP_EXPORT cvec zeros_c(int size);
 
 //! A Double (rows,cols)-matrix of zeros
-mat zeros(int rows, int cols);
+ITPP_EXPORT mat zeros(int rows, int cols);
 //! A Binary (rows,cols)-matrix of zeros
-bmat zeros_b(int rows, int cols);
+ITPP_EXPORT bmat zeros_b(int rows, int cols);
 //! A Int (rows,cols)-matrix of zeros
-imat zeros_i(int rows, int cols);
+ITPP_EXPORT imat zeros_i(int rows, int cols);
 //! A Double Complex (rows,cols)-matrix of zeros
-cmat zeros_c(int rows, int cols);
+ITPP_EXPORT cmat zeros_c(int rows, int cols);
 
 //! A Double (size,size) unit matrix
-mat eye(int size);
+ITPP_EXPORT mat eye(int size);
 //! A Binary (size,size) unit matrix
-bmat eye_b(int size);
+ITPP_EXPORT bmat eye_b(int size);
 //! A Int (size,size) unit matrix
-imat eye_i(int size);
+ITPP_EXPORT imat eye_i(int size);
 //! A Double Complex (size,size) unit matrix
-cmat eye_c(int size);
+ITPP_EXPORT cmat eye_c(int size);
 //! A non-copying version of the eye function.
 template <class T>
-void eye(int size, Mat<T> &m);
+void eye(int size, Mat<T> &m)
+{
+    m.set_size(size, size, false);
+    m = T(0);
+    for (int i = size - 1; i >= 0; i--)
+        m(i, i) = T(1);
+}
 
 //! Impulse vector
-vec impulse(int size);
-//! Linspace (works in the same way as the matlab version)
-vec linspace(double from, double to, int length = 100);
+ITPP_EXPORT vec impulse(int size);
+
+//! linspace (works in the same way as the MATLAB version)
+ITPP_EXPORT vec linspace(double from, double to, int length = 100);
+
+//! linspace_fixed_step (works in the same way as "from:step:to" in MATLAB)
+template<class T>
+Vec<T> linspace_fixed_step(T from, T to, T step = 1)
+{
+    int points = 0;
+    if (0 != step) {
+        points = itpp::floor_i(double(to-from)/step)+1;
+    }
+    if (0 >= points) {
+        return Vec<T>(0);
+    }
+
+    Vec<T> output(points);
+    output(0) = from;
+    for (int n = 1; n < points; ++n) {
+    	output(n) = output(n-1)+step;
+    }
+    return output;
+}
+
 /*! \brief Zig-zag space function (variation on linspace)
 
 This function is a variation on linspace().  It traverses the points
@@ -131,7 +159,7 @@ rather than from left to right (as does linspace).
 
 The result is a vector of length 1+2^K.
 */
-vec zigzag_space(double t0, double t1, int K = 5);
+ITPP_EXPORT vec zigzag_space(double t0, double t1, int K = 5);
 
 /*!
  * \brief Hadamard matrix
@@ -139,7 +167,7 @@ vec zigzag_space(double t0, double t1, int K = 5);
  * This function constructs a \a size by \a size Hadammard matrix, where
  * \a size is a power of 2.
  */
-imat hadamard(int size);
+ITPP_EXPORT imat hadamard(int size);
 
 /*!
   \brief Jacobsthal matrix.
@@ -157,7 +185,7 @@ imat hadamard(int size);
   See Wicker "Error Control Systems for digital communication and storage", p. 134
   for more information on these topics. Do not check that p is a prime.
 */
-imat jacobsthal(int p);
+ITPP_EXPORT imat jacobsthal(int p);
 
 /*!
   \brief Conference matrix.
@@ -172,7 +200,7 @@ imat jacobsthal(int p);
   For more details see pp. 55-58 in MacWilliams & Sloane "The theory of error correcting codes",
   North-Holland, 1977.
 */
-imat conference(int n);
+ITPP_EXPORT imat conference(int n);
 
 /*!
  * \brief Generate Toeplitz matrix from two vectors \c c and \c r.
@@ -200,46 +228,46 @@ imat conference(int n);
 template <typename Num_T>
 const Mat<Num_T> toeplitz(const Vec<Num_T> &c, const Vec<Num_T> &r)
 {
-  int n_rows = c.size();
-  int n_cols = r.size();
-  Mat<Num_T> output(n_rows, n_cols);
-  for (int i = 0; i < n_rows; ++i) {
-    int j_limit = std::min(n_cols, n_rows - i);
-    for (int j = 0; j < j_limit; ++j) {
-      output(i + j, j) = c(i);
+    int n_rows = c.size();
+    int n_cols = r.size();
+    Mat<Num_T> output(n_rows, n_cols);
+    for (int i = 0; i < n_rows; ++i) {
+        int j_limit = std::min(n_cols, n_rows - i);
+        for (int j = 0; j < j_limit; ++j) {
+            output(i + j, j) = c(i);
+        }
     }
-  }
-  for (int j = 1; j < n_cols; ++j) {
-    int i_limit = std::min(n_rows, n_cols - j);
-    for (int i = 0; i < i_limit; ++i) {
-      output(i, i + j) = r(j);
+    for (int j = 1; j < n_cols; ++j) {
+        int i_limit = std::min(n_rows, n_cols - j);
+        for (int i = 0; i < i_limit; ++i) {
+            output(i, i + j) = r(j);
+        }
     }
-  }
-  return output;
+    return output;
 }
 
 //! Generate symmetric Toeplitz matrix from vector \c c.
 template <typename Num_T>
 const Mat<Num_T> toeplitz(const Vec<Num_T> &c)
 {
-  int s = c.size();
-  Mat<Num_T> output(s, s);
-  for (int i = 0; i < s; ++i) {
-    for (int j = 0; j < s - i; ++j) {
-      output(i + j, j) = c(i);
+    int s = c.size();
+    Mat<Num_T> output(s, s);
+    for (int i = 0; i < s; ++i) {
+        for (int j = 0; j < s - i; ++j) {
+            output(i + j, j) = c(i);
+        }
     }
-  }
-  for (int j = 1; j < s; ++j) {
-    for (int i = 0; i < s - j; ++i) {
-      output(i, i + j) = c(j);
+    for (int j = 1; j < s; ++j) {
+        for (int i = 0; i < s - j; ++i) {
+            output(i, i + j) = c(j);
+        }
     }
-  }
-  return output;
+    return output;
 }
 
 //! Generate symmetric Toeplitz matrix from vector \c c (complex valued)
-template <>
-const cmat toeplitz(const cvec &c);
+ITPP_EXPORT const cmat toeplitz(const cvec &c);
+
 
 //!@}
 
@@ -248,43 +276,43 @@ const cmat toeplitz(const cvec &c);
   \brief Create a rotation matrix that rotates the given plane \c angle radians. Note that the order of the planes are important!
   \ingroup miscfunc
 */
-mat rotation_matrix(int dim, int plane1, int plane2, double angle);
+ITPP_EXPORT mat rotation_matrix(int dim, int plane1, int plane2, double angle);
 
 /*!
   \brief Calcualte the Householder vector
   \ingroup miscfunc
 */
-void house(const vec &x, vec &v, double &beta);
+ITPP_EXPORT void house(const vec &x, vec &v, double &beta);
 
 /*!
   \brief Calculate the Givens rotation values
   \ingroup miscfunc
 */
-void givens(double a, double b, double &c, double &s);
+ITPP_EXPORT void givens(double a, double b, double &c, double &s);
 
 /*!
   \brief Calculate the Givens rotation matrix
   \ingroup miscfunc
 */
-void givens(double a, double b, mat &m);
+ITPP_EXPORT void givens(double a, double b, mat &m);
 
 /*!
   \brief Calculate the Givens rotation matrix
   \ingroup miscfunc
 */
-mat givens(double a, double b);
+ITPP_EXPORT mat givens(double a, double b);
 
 /*!
   \brief Calculate the transposed Givens rotation matrix
   \ingroup miscfunc
 */
-void givens_t(double a, double b, mat &m);
+ITPP_EXPORT void givens_t(double a, double b, mat &m);
 
 /*!
   \brief Calculate the transposed Givens rotation matrix
   \ingroup miscfunc
 */
-mat givens_t(double a, double b);
+ITPP_EXPORT mat givens_t(double a, double b);
 
 /*!
   \relates Vec
@@ -293,9 +321,9 @@ mat givens_t(double a, double b);
 template <class T>
 Vec<T> vec_1(T v0)
 {
-  Vec<T> v(1);
-  v(0) = v0;
-  return v;
+    Vec<T> v(1);
+    v(0) = v0;
+    return v;
 }
 
 /*!
@@ -305,10 +333,10 @@ Vec<T> vec_1(T v0)
 template <class T>
 Vec<T> vec_2(T v0, T v1)
 {
-  Vec<T> v(2);
-  v(0) = v0;
-  v(1) = v1;
-  return v;
+    Vec<T> v(2);
+    v(0) = v0;
+    v(1) = v1;
+    return v;
 }
 
 /*!
@@ -318,11 +346,11 @@ Vec<T> vec_2(T v0, T v1)
 template <class T>
 Vec<T> vec_3(T v0, T v1, T v2)
 {
-  Vec<T> v(3);
-  v(0) = v0;
-  v(1) = v1;
-  v(2) = v2;
-  return v;
+    Vec<T> v(3);
+    v(0) = v0;
+    v(1) = v1;
+    v(2) = v2;
+    return v;
 }
 
 /*!
@@ -332,9 +360,9 @@ Vec<T> vec_3(T v0, T v1, T v2)
 template <class T>
 Mat<T> mat_1x1(T m00)
 {
-  Mat<T> m(1, 1);
-  m(0, 0) = m00;
-  return m;
+    Mat<T> m(1, 1);
+    m(0, 0) = m00;
+    return m;
 }
 
 /*!
@@ -344,10 +372,10 @@ Mat<T> mat_1x1(T m00)
 template <class T>
 Mat<T> mat_1x2(T m00, T m01)
 {
-  Mat<T> m(1, 2);
-  m(0, 0) = m00;
-  m(0, 1) = m01;
-  return m;
+    Mat<T> m(1, 2);
+    m(0, 0) = m00;
+    m(0, 1) = m01;
+    return m;
 }
 
 /*!
@@ -358,10 +386,10 @@ template <class T>
 Mat<T> mat_2x1(T m00,
                T m10)
 {
-  Mat<T> m(2, 1);
-  m(0, 0) = m00;
-  m(1, 0) = m10;
-  return m;
+    Mat<T> m(2, 1);
+    m(0, 0) = m00;
+    m(1, 0) = m10;
+    return m;
 }
 
 /*!
@@ -372,12 +400,12 @@ template <class T>
 Mat<T> mat_2x2(T m00, T m01,
                T m10, T m11)
 {
-  Mat<T> m(2, 2);
-  m(0, 0) = m00;
-  m(0, 1) = m01;
-  m(1, 0) = m10;
-  m(1, 1) = m11;
-  return m;
+    Mat<T> m(2, 2);
+    m(0, 0) = m00;
+    m(0, 1) = m01;
+    m(1, 0) = m10;
+    m(1, 1) = m11;
+    return m;
 }
 
 /*!
@@ -387,11 +415,11 @@ Mat<T> mat_2x2(T m00, T m01,
 template <class T>
 Mat<T> mat_1x3(T m00, T m01, T m02)
 {
-  Mat<T> m(1, 3);
-  m(0, 0) = m00;
-  m(0, 1) = m01;
-  m(0, 2) = m02;
-  return m;
+    Mat<T> m(1, 3);
+    m(0, 0) = m00;
+    m(0, 1) = m01;
+    m(0, 2) = m02;
+    return m;
 }
 
 /*!
@@ -403,11 +431,11 @@ Mat<T> mat_3x1(T m00,
                T m10,
                T m20)
 {
-  Mat<T> m(3, 1);
-  m(0, 0) = m00;
-  m(1, 0) = m10;
-  m(2, 0) = m20;
-  return m;
+    Mat<T> m(3, 1);
+    m(0, 0) = m00;
+    m(1, 0) = m10;
+    m(2, 0) = m20;
+    return m;
 }
 
 /*!
@@ -418,14 +446,14 @@ template <class T>
 Mat<T> mat_2x3(T m00, T m01, T m02,
                T m10, T m11, T m12)
 {
-  Mat<T> m(2, 3);
-  m(0, 0) = m00;
-  m(0, 1) = m01;
-  m(0, 2) = m02;
-  m(1, 0) = m10;
-  m(1, 1) = m11;
-  m(1, 2) = m12;
-  return m;
+    Mat<T> m(2, 3);
+    m(0, 0) = m00;
+    m(0, 1) = m01;
+    m(0, 2) = m02;
+    m(1, 0) = m10;
+    m(1, 1) = m11;
+    m(1, 2) = m12;
+    return m;
 }
 
 /*!
@@ -437,14 +465,14 @@ Mat<T> mat_3x2(T m00, T m01,
                T m10, T m11,
                T m20, T m21)
 {
-  Mat<T> m(3, 2);
-  m(0, 0) = m00;
-  m(0, 1) = m01;
-  m(1, 0) = m10;
-  m(1, 1) = m11;
-  m(2, 0) = m20;
-  m(2, 1) = m21;
-  return m;
+    Mat<T> m(3, 2);
+    m(0, 0) = m00;
+    m(0, 1) = m01;
+    m(1, 0) = m10;
+    m(1, 1) = m11;
+    m(2, 0) = m20;
+    m(2, 1) = m21;
+    return m;
 }
 
 /*!
@@ -456,17 +484,17 @@ Mat<T> mat_3x3(T m00, T m01, T m02,
                T m10, T m11, T m12,
                T m20, T m21, T m22)
 {
-  Mat<T> m(3, 3);
-  m(0, 0) = m00;
-  m(0, 1) = m01;
-  m(0, 2) = m02;
-  m(1, 0) = m10;
-  m(1, 1) = m11;
-  m(1, 2) = m12;
-  m(2, 0) = m20;
-  m(2, 1) = m21;
-  m(2, 2) = m22;
-  return m;
+    Mat<T> m(3, 3);
+    m(0, 0) = m00;
+    m(0, 1) = m01;
+    m(0, 2) = m02;
+    m(1, 0) = m10;
+    m(1, 1) = m11;
+    m(1, 2) = m12;
+    m(2, 0) = m20;
+    m(2, 1) = m21;
+    m(2, 2) = m22;
+    return m;
 }
 
 } //namespace itpp
