@@ -5,24 +5,23 @@
  *
  * -------------------------------------------------------------------------
  *
- * IT++ - C++ library of mathematical, signal processing, speech processing,
- *        and communications classes and functions
+ * Copyright (C) 1995-2010  (see AUTHORS file for a list of contributors)
  *
- * Copyright (C) 1995-2009  (see AUTHORS file for a list of contributors)
+ * This file is part of IT++ - a C++ library of mathematical, signal
+ * processing, speech processing, and communications classes and functions.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * IT++ is free software: you can redistribute it and/or modify it under the
+ * terms of the GNU General Public License as published by the Free Software
+ * Foundation, either version 3 of the License, or (at your option) any
+ * later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * IT++ is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ * You should have received a copy of the GNU General Public License along
+ * with IT++.  If not, see <http://www.gnu.org/licenses/>.
  *
  * -------------------------------------------------------------------------
  */
@@ -91,7 +90,7 @@ void CRC_Code::parity(const bvec &in_bits, bvec &out) const
 
   for (int i = 0; i < temp.size() - polynomial.size() + 1; i++) {
     if (temp(i) == 1) {
-      temp.set_subvector(i, i + no_parity, temp(i, i + no_parity) + polynomial);
+      temp.set_subvector(i, temp(i, i + no_parity) + polynomial);
     }
   }
 
@@ -118,7 +117,7 @@ bool CRC_Code::check_parity(const bvec &coded_bits) const
 
   for (int i = 0; i < temp.size() - polynomial.size() + 1; i++) {
     if (temp(i) == 1) {
-      temp.set_subvector(i, i + no_parity, temp(i, i + no_parity) + polynomial);
+      temp.set_subvector(i, temp(i, i + no_parity) + polynomial);
     }
   }
 
