@@ -928,7 +928,6 @@ ivec LDPC_Generator_Systematic::construct(LDPC_Parity* const H,
   it_info_debug("Computing a systematic generator matrix...");
 
   int j1 = 0, j2 = 0;
-  int rank;
   ivec perm;
   GF2mat T, U;
   for (int k = 0; k < nvar; k++) {
@@ -938,7 +937,7 @@ ivec LDPC_Generator_Systematic::construct(LDPC_Parity* const H,
     bvec c = Hd.get_col(col_order(k));
     if (j2 == 0) {     // first column in P2 is number col_order(k)
       P2 = GF2mat(c);
-      rank = P2.T_fact(T, U, perm);
+      P2.T_fact(T, U, perm);/* return value not used */
       actual_ordering(k) = nvar - ncheck;
       j2++;
     }
@@ -988,7 +987,7 @@ ivec LDPC_Generator_Systematic::construct(LDPC_Parity* const H,
   G = G.transpose();  // store the generator matrix in transposed form
   it_info_debug("Systematic generator matrix computed.");
 
-  init_flag = true;
+  mark_initialized();
 
   return actual_ordering;
 }
@@ -1001,7 +1000,7 @@ void LDPC_Generator_Systematic::save(const std::string& filename) const
   f >> Name("Fileversion") >> ver;
   it_assert(ver == LDPC_binary_file_version,
             "LDPC_Generator_Systematic::save(): Unsupported file format");
-  f << Name("G_type") << type;
+  f << Name("G_type") << get_type();
   f << Name("G") << G;
   f.close();
 }
@@ -1016,18 +1015,18 @@ void LDPC_Generator_Systematic::load(const std::string& filename)
             "LDPC_Generator_Systematic::load(): Unsupported file format");
   std::string gen_type;
   f >> Name("G_type") >> gen_type;
-  it_assert(gen_type == type,
+  it_assert(gen_type == get_type(),
             "LDPC_Generator_Systematic::load(): Wrong generator type");
   f >> Name("G") >> G;
   f.close();
 
-  init_flag = true;
+  mark_initialized();
 }
 
 
 void LDPC_Generator_Systematic::encode(const bvec &input, bvec &output)
 {
-  it_assert(init_flag, "LDPC_Generator_Systematic::encode(): Systematic "
+  it_assert(is_initialized(), "LDPC_Generator_Systematic::encode(): Systematic "
             "generator not set up");
   it_assert(input.size() == G.cols(), "LDPC_Generator_Systematic::encode(): "
             "Improper input vector size (" << input.size() << " != "
@@ -1051,7 +1050,7 @@ BLDPC_Generator::BLDPC_Generator(const BLDPC_Parity* const H,
 
 void BLDPC_Generator::encode(const bvec &input, bvec &output)
 {
-  it_assert(init_flag, "BLDPC_Generator::encode(): Cannot encode with not "
+  it_assert(is_initialized(), "BLDPC_Generator::encode(): Cannot encode with not "
             "initialized generator");
   it_assert(input.size() == K, "BLDPC_Generator::encode(): Input vector "
             "length is not equal to K");
@@ -1133,14 +1132,14 @@ void BLDPC_Generator::construct(const BLDPC_Parity* const H)
       r1++;
     }
 
-    init_flag = true;
+    mark_initialized();
   }
 }
 
 
 void BLDPC_Generator::save(const std::string& filename) const
 {
-  it_assert(init_flag,
+  it_assert(is_initialized(),
             "BLDPC_Generator::save(): Can not save not initialized generator");
   // Every Z-th row of H_enc until M-Z
   GF2mat H_T(M / Z - 1, N);
@@ -1155,7 +1154,7 @@ void BLDPC_Generator::save(const std::string& filename) const
   f >> Name("Fileversion") >> ver;
   it_assert(ver == LDPC_binary_file_version, "BLDPC_Generator::save(): "
             "Unsupported file format");
-  f << Name("G_type") << type;
+  f << Name("G_type") << get_type();
   f << Name("H_T") << H_T;
   f << Name("H_Z") << H_Z;
   f << Name("Z") << Z;
@@ -1173,7 +1172,7 @@ void BLDPC_Generator::load(const std::string& filename)
             "Unsupported file format");
   std::string gen_type;
   f >> Name("G_type") >> gen_type;
-  it_assert(gen_type == type,
+  it_assert(gen_type == get_type(),
             "BLDPC_Generator::load(): Wrong generator type");
   f >> Name("H_T") >> H_T;
   f >> Name("H_Z") >> H_Z;
@@ -1196,7 +1195,7 @@ void BLDPC_Generator::load(const std::string& filename)
   }
   H_enc = H_enc.concatenate_vertical(H_Z);
 
-  init_flag = true;
+  mark_initialized();
 }
 
 
@@ -1204,36 +1203,44 @@ void BLDPC_Generator::load(const std::string& filename)
 // LDPC_Code
 // ----------------------------------------------------------------------
 
-LDPC_Code::LDPC_Code(): H_defined(false), G_defined(false), dec_method("BP"),
+LDPC_Code::LDPC_Code(): H_defined(false), G_defined(false), dec_method(new std::string),
     max_iters(50), psc(true), pisc(false),
-    llrcalc(LLR_calc_unit()) { }
+    llrcalc(LLR_calc_unit()) { set_decoding_method("BP");}
 
 LDPC_Code::LDPC_Code(const LDPC_Parity* const H,
-                     LDPC_Generator* const G_in):
-    H_defined(false), G_defined(false), dec_method("BP"), max_iters(50),
+                     LDPC_Generator* const G_in,
+                     bool perform_integrity_check):
+    H_defined(false), G_defined(false), dec_method(new std::string), max_iters(50),
     psc(true), pisc(false), llrcalc(LLR_calc_unit())
 {
-  set_code(H, G_in);
+    set_decoding_method("BP");
+    set_code(H, G_in, perform_integrity_check);
 }
 
 LDPC_Code::LDPC_Code(const std::string& filename,
                      LDPC_Generator* const G_in):
-    H_defined(false), G_defined(false), dec_method("BP"), max_iters(50),
+    H_defined(false), G_defined(false), dec_method(new std::string), max_iters(50),
     psc(true), pisc(false), llrcalc(LLR_calc_unit())
 {
+  set_decoding_method("BP");
   load_code(filename, G_in);
 }
 
 
 void LDPC_Code::set_code(const LDPC_Parity* const H,
-                         LDPC_Generator* const G_in)
+                         LDPC_Generator* const G_in,
+                         bool perform_integrity_check)
 {
   decoder_parameterization(H);
   setup_decoder();
   G = G_in;
   if (G != 0) {
     G_defined = true;
-    integrity_check();
+    if (perform_integrity_check) {
+      integrity_check();
+    } else {
+      it_info_debug("LDPC_Code::set_code(): integrity check was not performed");
+    }
   }
 }
 
@@ -1317,7 +1324,7 @@ void LDPC_Code::set_decoding_method(const std::string& method_in)
 {
   it_assert((method_in == "bp") || (method_in == "BP"),
             "LDPC_Code::set_decoding_method(): Not implemented decoding method");
-  dec_method = method_in;
+  *dec_method = method_in;
 }
 
 void LDPC_Code::set_exit_conditions(int max_iters_in,
@@ -1551,10 +1558,12 @@ int LDPC_Code::bp_decode(const QLLRvec &LLRin, QLLRvec &LLRout)
       case 0:
         it_error("LDPC_Code::bp_decode(): sumX1(i)=0");
       case 1: {
-        // Degenerate case-should not occur for good coded. A lonely
-        // variable node only sends its incoming message
+        /* This case is rare but apparently occurs for codes used in
+	   the DVB-T2 standard.
+	 */
+	QLLR m0 = mcv[iind[i]];
         mvc[i] = LLRin(i);
-        LLRout(i) = LLRin(i);
+        LLRout(i) = LLRin(i) + m0;
         break;
       }
       case 2: {
@@ -1653,6 +1662,23 @@ bool LDPC_Code::syndrome_check(const QLLRvec &LLR) const
   return true;   // codeword is valid
 };
 
+QLLRvec LDPC_Code::soft_syndrome_check(const QLLRvec &LLR) const
+{
+  QLLRvec result(ncheck);
+  int i,j,vi,vind;
+
+  for (j=0; j<ncheck; j++) {
+    result(j)=-QLLR_MAX;
+    vind=j;
+    for (i=0; i<sumX2(j); i++) {
+      vi=V(vind);
+      result(j) = llrcalc.Boxplus(LLR(vi),result(j));
+      vind += ncheck;
+    }
+  }
+
+  return result;
+}
 
 // ----------------------------------------------------------------------
 // LDPC_Code private methods
@@ -1782,7 +1808,7 @@ std::ostream &operator<<(std::ostream &os, const LDPC_Code &C)
   << "Row degrees (node perspective): " << rdeg << "\n"
   << "-------------------------------------------------\n"
   << "Decoder parameters:\n"
-  << " - method : " << C.dec_method << "\n"
+  << " - method : " << C.get_decoding_method() << "\n"
   << " - max. iterations : " << C.max_iters << "\n"
   << " - syndrome check at each iteration : " << C.psc << "\n"
   << " - syndrome check at start : " << C.pisc << "\n"
