@@ -31,9 +31,29 @@
 
 namespace itpp
 {
-
+// -------------------------------------------------------------------------------------
+// Turbo Codec
+// -------------------------------------------------------------------------------------
+std::string Turbo_Codec::string_from_metric(const Turbo_Codec::Metric& in_metric)
+{
+  if(in_metric == Metric::LOGMAX) {
+    return std::string("LOGMAX");
+  }
+  else if(in_metric == Metric::LOGMAP) {
+    return std::string("LOGMAP");
+  }
+  else if(in_metric == Metric::MAP) {
+    return std::string("MAP");
+  }
+  else if(in_metric == Metric::TABLE) {
+    return std::string("TABLE");
+  }
+  else {
+    return std::string("UNKNOWN");
+  }
+}
 void Turbo_Codec::set_parameters(ivec gen1, ivec gen2, int constraint_length, const ivec &interleaver_sequence,
-                                 int in_iterations, std::string in_metric, double in_logmax_scale_factor,
+                                 int in_iterations, const std::string &in_metric, double in_logmax_scale_factor,
                                  bool in_adaptive_stop,  LLR_calc_unit in_llrcalc)
 {
   //Set the input parameters:
@@ -44,24 +64,24 @@ void Turbo_Codec::set_parameters(ivec gen1, ivec gen2, int constraint_length, co
   adaptive_stop       = in_adaptive_stop;
 
   //Check the decoding metric
-  if (in_metric == "LOGMAX") {
-    metric = "LOGMAX";
+  if(in_metric == "LOGMAX") {
+    metric = Metric::LOGMAX;
   }
-  else if (in_metric == "LOGMAP") {
-    metric = "LOGMAP";
+  else if(in_metric == "LOGMAP") {
+    metric = Metric::LOGMAP;
   }
-  else if (in_metric == "MAP") {
-    metric = "MAP";
+  else if(in_metric == "MAP") {
+    metric = Metric::MAP;
   }
-  else if (in_metric == "TABLE") {
-    metric = "TABLE";
+  else if(in_metric == "TABLE") {
+    metric = Metric::TABLE;
   }
   else {
-    it_error("Turbo_Codec::set_parameters: The decoder metric must be either MAP, LOGMAP or LOGMAX");
+    it_error("Turbo_Codec::set_parameters: The decoder metric must be either MAP, LOGMAP, LOGMAX or TABLE");
   }
 
-  if (logmax_scale_factor != 1.0) {
-    it_assert(metric == "LOGMAX", "Turbo_Codec::set_parameters: logmax_scale_factor can only be used together with LOGMAX decoding");
+  if(logmax_scale_factor != 1.0) {
+    it_assert(metric == Metric::LOGMAX, "Turbo_Codec::set_parameters: logmax_scale_factor can only be used together with LOGMAX decoding");
   }
 
   //The RSC Encoders:
@@ -112,20 +132,20 @@ void Turbo_Codec::set_metric(std::string in_metric, double in_logmax_scale_facto
   logmax_scale_factor = in_logmax_scale_factor;
 
   //Check the decoding metric
-  if (in_metric == "LOGMAX") {
-    metric = "LOGMAX";
+  if(in_metric == "LOGMAX") {
+    metric = Metric::LOGMAX;
   }
-  else if (in_metric == "LOGMAP") {
-    metric = "LOGMAP";
+  else if(in_metric == "LOGMAP") {
+    metric = Metric::LOGMAP;
   }
-  else if (in_metric == "MAP") {
-    metric = "MAP";
+  else if(in_metric == "MAP") {
+    metric = Metric::MAP;
   }
-  else if (in_metric == "TABLE") {
-    metric = "TABLE";
+  else if(in_metric == "TABLE") {
+    metric = Metric::TABLE;
   }
   else {
-    it_error("Turbo_Codec::set_metric: The decoder metric must be either MAP, LOGMAP or LOGMAX");
+    it_error("Turbo_Codec::set_metric: The decoder metric must be either MAP, LOGMAP, LOGMAX or TABLE");
   }
 
   rscc1.set_llrcalc(in_llrcalc);
@@ -165,38 +185,38 @@ void Turbo_Codec::encode(const bvec &input, bvec &output)
 
   //Initializations:
   no_blocks = input.length() / Nuncoded;
-  output.set_size(no_blocks*Ncoded, false);
+  output.set_size(no_blocks * Ncoded, false);
 
   //Set the bit counter to zero:
   count = 0;
 
   //Encode all code blocks:
-  for (i = 0; i < no_blocks; i++) {
+  for(i = 0; i < no_blocks; i++) {
 
     //Encode one block
     input_bits = input.mid(i * Nuncoded, Nuncoded);
     encode_block(input_bits, in1, in2, parity1, parity2);
 
     //The data part:
-    for (k = 0; k < Nuncoded; k++) {
+    for(k = 0; k < Nuncoded; k++) {
       output(count) = in1(k);
       count++;                                //Systematic bits
-      for (j = 0; j < n1; j++) { output(count) = parity1(k, j); count++; } //Parity-1 bits
-      for (j = 0; j < n2; j++) { output(count) = parity2(k, j); count++; } //Parity-2 bits
+      for(j = 0; j < n1; j++) { output(count) = parity1(k, j); count++; }  //Parity-1 bits
+      for(j = 0; j < n2; j++) { output(count) = parity2(k, j); count++; }  //Parity-2 bits
     }
 
     //The first tail:
-    for (k = 0; k < m_tail; k++) {
+    for(k = 0; k < m_tail; k++) {
       output(count) = in1(Nuncoded + k);
       count++;                                //First systematic tail bit
-      for (j = 0; j < n1; j++) { output(count) = parity1(Nuncoded + k, j); count++; } //Parity-1 tail bits
+      for(j = 0; j < n1; j++) { output(count) = parity1(Nuncoded + k, j); count++; }  //Parity-1 tail bits
     }
 
     //The second tail:
-    for (k = 0; k < m_tail; k++) {
+    for(k = 0; k < m_tail; k++) {
       output(count) = in2(Nuncoded + k);
       count++;                                //Second systematic tail bit
-      for (j = 0; j < n2; j++) { output(count) = parity2(Nuncoded + k, j); count++; } //Parity-2 tail bits
+      for(j = 0; j < n2; j++) { output(count) = parity2(Nuncoded + k, j); count++; }  //Parity-2 tail bits
     }
 
   }
@@ -213,7 +233,7 @@ void Turbo_Codec::decode(const vec &received_signal, bvec &decoded_bits, ivec &n
                          const bvec &true_bits)
 {
 
-  if ((n1 == 1) && (n2 == 1) && (metric != "MAP")) {
+  if((n1 == 1) && (n2 == 1) && (metric != Metric::MAP)) {
     //This is a speed optimized decoder for R=1/3 (log domain metrics only)
     decode_n3(received_signal, decoded_bits, nrof_used_iterations, true_bits);
   }
@@ -239,7 +259,7 @@ void Turbo_Codec::decode(const vec &received_signal, bvec &decoded_bits, ivec &n
     nrof_used_iterations.set_size(no_blocks, false);
 
     //Check the vector true_bits:
-    if (true_bits.size() > 1) {
+    if(true_bits.size() > 1) {
       it_assert(true_bits.size() == (Nuncoded * no_blocks), "Turbo_Codec::decode: Wrong size of input vectors");
       CHECK_TRUE_BITS = true;
     }
@@ -251,32 +271,32 @@ void Turbo_Codec::decode(const vec &received_signal, bvec &decoded_bits, ivec &n
     count = 0;
 
     //Itterate over all received code blocks:
-    for (i = 0; i < no_blocks; i++) {
+    for(i = 0; i < no_blocks; i++) {
 
       //The data part:
-      for (k = 0; k < Nuncoded; k++) {
+      for(k = 0; k < Nuncoded; k++) {
         rec_syst1(k) = received_signal(count);
         count++;                               //Systematic bit
-        for (j = 0; j < n1; j++) { rec_parity1(k, j) = received_signal(count); count++; }  //Parity-1 bits
-        for (j = 0; j < n2; j++) { rec_parity2(k, j) = received_signal(count); count++; }  //Parity-2 bits
+        for(j = 0; j < n1; j++) { rec_parity1(k, j) = received_signal(count); count++; }   //Parity-1 bits
+        for(j = 0; j < n2; j++) { rec_parity2(k, j) = received_signal(count); count++; }   //Parity-2 bits
       }
 
       //The first tail:
-      for (k = 0; k < m_tail; k++) {
+      for(k = 0; k < m_tail; k++) {
         rec_syst1(Nuncoded + k) = received_signal(count);
         count++;                               //Tail 1 systematic bit
-        for (j = 0; j < n1; j++) { rec_parity1(Nuncoded + k, j) = received_signal(count); count++; }  //Tail 1 parity-1 bits
+        for(j = 0; j < n1; j++) { rec_parity1(Nuncoded + k, j) = received_signal(count); count++; }   //Tail 1 parity-1 bits
       }
 
       //The second tail:
-      for (k = 0; k < m_tail; k++) {
+      for(k = 0; k < m_tail; k++) {
         rec_syst2(Nuncoded + k) = received_signal(count);
         count++;                              //Tail2 systematic bit
-        for (j = 0; j < n2; j++) { rec_parity2(Nuncoded + k, j) = received_signal(count); count++; } //Tali2 parity-2 bits
+        for(j = 0; j < n2; j++) { rec_parity2(Nuncoded + k, j) = received_signal(count); count++; }  //Tali2 parity-2 bits
       }
 
       //Scale the input data if necessary:
-      if (Lc != 1.0) {
+      if(Lc != 1.0) {
         rec_syst1 *= Lc;
         rec_syst2 *= Lc;
         rec_parity1 *= Lc;
@@ -284,9 +304,9 @@ void Turbo_Codec::decode(const vec &received_signal, bvec &decoded_bits, ivec &n
       }
 
       //Decode the block:
-      if (CHECK_TRUE_BITS) {
+      if(CHECK_TRUE_BITS) {
         decode_block(rec_syst1, rec_syst2, rec_parity1, rec_parity2, decoded_bits_i,
-                     nrof_used_iterations_i, true_bits.mid(i*Nuncoded, Nuncoded));
+                     nrof_used_iterations_i, true_bits.mid(i * Nuncoded, Nuncoded));
         nrof_used_iterations(i) = nrof_used_iterations_i;
       }
       else {
@@ -295,7 +315,7 @@ void Turbo_Codec::decode(const vec &received_signal, bvec &decoded_bits, ivec &n
       }
 
       //Put the decoded bits in the output vector:
-      decoded_bits.replace_mid(i*Nuncoded, decoded_bits_i.get_row(iterations - 1));
+      decoded_bits.replace_mid(i * Nuncoded, decoded_bits_i.get_row(iterations - 1));
 
     }
 
@@ -372,7 +392,7 @@ void Turbo_Codec::decode_block(const vec &rec_syst1, const vec &rec_syst2, const
   int_rec_syst = concat(int_rec_syst, tail2);
 
   // Check the vector true_bits
-  if (true_bits.size() > 1) {
+  if(true_bits.size() > 1) {
     it_assert(true_bits.size() == Nuncoded, "Turbo_Codec::decode_block: Illegal size of input vector true_bits");
     CHECK_TRUE_BITS = true;
   }
@@ -380,22 +400,22 @@ void Turbo_Codec::decode_block(const vec &rec_syst1, const vec &rec_syst2, const
     CHECK_TRUE_BITS = false;
   }
 
-  if (CHECK_TRUE_BITS) {
+  if(CHECK_TRUE_BITS) {
     it_assert(adaptive_stop == false,
               "Turbo_Codec::decode_block: You can not stop iterations both adaptively and on true bits");
   }
 
   // Do the iterative decoding:
   nrof_used_iterations_i = iterations;
-  for (i = 0; i < iterations; i++) {
+  for(i = 0; i < iterations; i++) {
 
     // Decode Code 1
-    if (metric == "MAP") {
+    if(metric == Metric::MAP) {
       rscc1.map_decode(rec_syst, rec_parity1, Le21, Le12, true);
     }
-    else if ((metric == "LOGMAX") || (metric == "LOGMAP") || (metric == "TABLE")) {
-      rscc1.log_decode(rec_syst, rec_parity1, Le21, Le12, true, metric);
-      if (logmax_scale_factor != 1.0) {
+    else if((metric == Metric::LOGMAX) || (metric == Metric::LOGMAP) || (metric == Metric::TABLE)) {
+      rscc1.log_decode(rec_syst, rec_parity1, Le21, Le12, true, string_from_metric(metric));
+      if(logmax_scale_factor != 1.0) {
         Le12 *= logmax_scale_factor;
       }
     }
@@ -408,12 +428,12 @@ void Turbo_Codec::decode_block(const vec &rec_syst1, const vec &rec_syst2, const
     Le12_int = concat(tmp, zeros(Le12.size() - interleaver_size));
 
     // Decode Code 2
-    if (metric == "MAP") {
+    if(metric == Metric::MAP) {
       rscc2.map_decode(int_rec_syst, rec_parity2, Le12_int, Le21_int, true);
     }
-    else if ((metric == "LOGMAX") || (metric == "LOGMAP")  || (metric == "TABLE")) {
-      rscc2.log_decode(int_rec_syst, rec_parity2, Le12_int, Le21_int, true, metric);
-      if (logmax_scale_factor != 1.0) {
+    else if((metric == Metric::LOGMAX) || (metric == Metric::LOGMAP) || (metric == Metric::TABLE)) {
+      rscc2.log_decode(int_rec_syst, rec_parity2, Le12_int, Le21_int, true, string_from_metric(metric));
+      if(logmax_scale_factor != 1.0) {
         Le21_int *= logmax_scale_factor;
       }
     }
@@ -428,31 +448,31 @@ void Turbo_Codec::decode_block(const vec &rec_syst1, const vec &rec_syst2, const
     // Take bit decisions
     L = rec_syst + Le21 + Le12;
     count = 0;
-    for (l = 0; l < Nuncoded; l++) {
+    for(l = 0; l < Nuncoded; l++) {
       (L(l) > 0.0) ? (decoded_bits_i(i, count) = bin(0)) : (decoded_bits_i(i, count) = bin(1));
       count++;
     }
 
     //Check if it is possible to stop iterating early:
     CONTINUE = true;
-    if (i < (iterations - 1)) {
+    if(i < (iterations - 1)) {
 
-      if (CHECK_TRUE_BITS) {
+      if(CHECK_TRUE_BITS) {
         CONTINUE = false;
-        for (k = 0; k < Nuncoded; k++) { if (true_bits(k) != decoded_bits_i(i, k)) { CONTINUE = true; break; } }
+        for(k = 0; k < Nuncoded; k++) { if(true_bits(k) != decoded_bits_i(i, k)) { CONTINUE = true; break; } }
       }
 
-      if ((adaptive_stop) && (i > 0)) {
+      if((adaptive_stop) && (i > 0)) {
         CONTINUE = false;
-        for (k = 0; k < Nuncoded; k++) { if (decoded_bits_i(i - 1, k) != decoded_bits_i(i, k)) { CONTINUE = true; break; } }
+        for(k = 0; k < Nuncoded; k++) { if(decoded_bits_i(i - 1, k) != decoded_bits_i(i, k)) { CONTINUE = true; break; } }
       }
 
     }
 
     //Check if iterations shall continue:
-    if (CONTINUE == false) {
+    if(CONTINUE == false) {
       //Copy the results from current iteration to all following iterations:
-      for (k = (i + 1); k < iterations; k++) {
+      for(k = (i + 1); k < iterations; k++) {
         decoded_bits_i.set_row(k, decoded_bits_i.get_row(i));
         nrof_used_iterations_i = i + 1;
       }
@@ -496,27 +516,27 @@ void Turbo_Codec::decode_n3(const vec &received_signal, bvec &decoded_bits, ivec
   count_out = 0;
 
   // Check the vector true_bits
-  if (true_bits.size() > 1) {
-    it_assert(true_bits.size() == Nuncoded*no_blocks, "Turbo_Codec::decode_n3: Illegal size of input vector true_bits");
+  if(true_bits.size() > 1) {
+    it_assert(true_bits.size() == Nuncoded * no_blocks, "Turbo_Codec::decode_n3: Illegal size of input vector true_bits");
     CHECK_TRUE_BITS = true;
   }
   else {
     CHECK_TRUE_BITS = false;
   }
 
-  if (CHECK_TRUE_BITS) {
+  if(CHECK_TRUE_BITS) {
     it_assert(adaptive_stop == false,
               "Turbo_Codec::decode_block: You can not stop iterations both adaptively and on true bits");
   }
 
   //Iterate over all received code blocks:
-  for (i = 0; i < no_blocks; i++) {
+  for(i = 0; i < no_blocks; i++) {
 
     //Reset extrinsic data:
     Le21.zeros();
 
     //The data part:
-    for (k = 0; k < Nuncoded; k++) {
+    for(k = 0; k < Nuncoded; k++) {
       rec_syst1(k)   = received_signal(count);
       count++; //Systematic bit
       rec_parity1(k) = received_signal(count);
@@ -526,7 +546,7 @@ void Turbo_Codec::decode_n3(const vec &received_signal, bvec &decoded_bits, ivec
     }
 
     //The first tail:
-    for (k = 0; k < m_tail; k++) {
+    for(k = 0; k < m_tail; k++) {
       rec_syst1(Nuncoded + k)   = received_signal(count);
       count++; //Tail 1 systematic bit
       rec_parity1(Nuncoded + k) = received_signal(count);
@@ -534,7 +554,7 @@ void Turbo_Codec::decode_n3(const vec &received_signal, bvec &decoded_bits, ivec
     }
 
     //The second tail:
-    for (k = 0; k < m_tail; k++) {
+    for(k = 0; k < m_tail; k++) {
       rec_syst2(Nuncoded + k)   = received_signal(count);
       count++; //Tail2 systematic bit
       rec_parity2(Nuncoded + k) = received_signal(count);
@@ -545,7 +565,7 @@ void Turbo_Codec::decode_n3(const vec &received_signal, bvec &decoded_bits, ivec
     rec_syst2.replace_mid(0, int_rec_syst1);
 
     //Scale the input data if necessary:
-    if (Lc != 1.0) {
+    if(Lc != 1.0) {
       rec_syst1   *= Lc;
       rec_syst2   *= Lc;
       rec_parity1 *= Lc;
@@ -555,45 +575,45 @@ void Turbo_Codec::decode_n3(const vec &received_signal, bvec &decoded_bits, ivec
     //Decode the block:
     CONTINUE = true;
     nrof_used_iterations_i = iterations;
-    for (j = 0; j < iterations; j++) {
+    for(j = 0; j < iterations; j++) {
 
-      rscc1.log_decode_n2(rec_syst1, rec_parity1, Le21, Le12, true, metric);
-      if (logmax_scale_factor != 1.0) { Le12 *= logmax_scale_factor; }
+      rscc1.log_decode_n2(rec_syst1, rec_parity1, Le21, Le12, true, string_from_metric(metric));
+      if(logmax_scale_factor != 1.0) { Le12 *= logmax_scale_factor; }
       float_interleaver.interleave(Le12, Le12_int);
 
-      rscc2.log_decode_n2(rec_syst2, rec_parity2, Le12_int, Le21_int, true, metric);
-      if (logmax_scale_factor != 1.0) { Le21_int *= logmax_scale_factor; }
+      rscc2.log_decode_n2(rec_syst2, rec_parity2, Le12_int, Le21_int, true, string_from_metric(metric));
+      if(logmax_scale_factor != 1.0) { Le21_int *= logmax_scale_factor; }
       float_interleaver.deinterleave(Le21_int, Le21);
 
-      if (adaptive_stop) {
+      if(adaptive_stop) {
         L = rec_syst1.left(Nuncoded) + Le21.left(Nuncoded) + Le12.left(Nuncoded);
-        for (l = 0; l < Nuncoded; l++) {(L(l) > 0.0) ? (temp_decoded_bits(l) = bin(0)) : (temp_decoded_bits(l) = bin(1)); }
-        if (j == 0) { decoded_bits_previous_iteration = temp_decoded_bits; }
+        for(l = 0; l < Nuncoded; l++) {(L(l) > 0.0) ? (temp_decoded_bits(l) = bin(0)) : (temp_decoded_bits(l) = bin(1)); }
+        if(j == 0) { decoded_bits_previous_iteration = temp_decoded_bits; }
         else {
-          if (temp_decoded_bits == decoded_bits_previous_iteration) {
+          if(temp_decoded_bits == decoded_bits_previous_iteration) {
             CONTINUE = false;
           }
-          else if (j < (iterations - 1)) {
+          else if(j < (iterations - 1)) {
             decoded_bits_previous_iteration = temp_decoded_bits;
           }
         }
       }
 
-      if (CHECK_TRUE_BITS) {
+      if(CHECK_TRUE_BITS) {
         L = rec_syst1.left(Nuncoded) + Le21.left(Nuncoded) + Le12.left(Nuncoded);
-        for (l = 0; l < Nuncoded; l++) {(L(l) > 0.0) ? (temp_decoded_bits(l) = bin(0)) : (temp_decoded_bits(l) = bin(1)); }
-        if (temp_decoded_bits == true_bits.mid(i*Nuncoded, Nuncoded)) {
+        for(l = 0; l < Nuncoded; l++) {(L(l) > 0.0) ? (temp_decoded_bits(l) = bin(0)) : (temp_decoded_bits(l) = bin(1)); }
+        if(temp_decoded_bits == true_bits.mid(i * Nuncoded, Nuncoded)) {
           CONTINUE = false;
         }
       }
 
-      if (CONTINUE == false) { nrof_used_iterations_i = j + 1; break; }
+      if(CONTINUE == false) { nrof_used_iterations_i = j + 1; break; }
 
     }
 
     //Take final bit decisions
     L = rec_syst1.left(Nuncoded) + Le21.left(Nuncoded) + Le12.left(Nuncoded);
-    for (l = 0; l < Nuncoded; l++) {
+    for(l = 0; l < Nuncoded; l++) {
       (L(l) > 0.0) ? (decoded_bits(count_out) = bin(0)) : (decoded_bits(count_out) = bin(1));
       count_out++;
     }
@@ -604,6 +624,312 @@ void Turbo_Codec::decode_n3(const vec &received_signal, bvec &decoded_bits, ivec
 
 }
 
+// -------------------------------------------------------------------------------------
+// Punctured Turbo Codec
+// -------------------------------------------------------------------------------------
+
+void Punctured_Turbo_Codec::set_parameters(ivec gen1, ivec gen2, int constraint_length, const ivec &interleaver_sequence, bmat &pmatrix, int in_iterations, std::string in_metric, double in_logmax_scale_factor, bool in_adaptive_stop, itpp::LLR_calc_unit lcalc)
+{
+  Turbo_Codec::set_parameters(gen1, gen2, constraint_length, interleaver_sequence, in_iterations, in_metric, in_logmax_scale_factor, in_adaptive_stop, lcalc);
+  set_puncture_matrix(pmatrix);
+}
+
+void Punctured_Turbo_Codec::set_puncture_matrix(const bmat &pmatrix)
+{
+  int p, j;
+
+  punct_total = 0;
+  punct_total2 = 0;
+
+  it_error_if(pmatrix.rows() != n_tot || pmatrix.cols() == 0, "Wrong size of puncture matrix");
+  puncture_matrix = pmatrix;
+  Period = puncture_matrix.cols();
+
+  // all rows
+  for(j = 0; j < n_tot; j++) {
+    for(p = 0; p < Period; p++)
+      punct_total += static_cast<int>(puncture_matrix(j, p));
+  }
+  // systematic bits
+  for(p = 0; p < Period; p++)
+    punct_total2 += static_cast<int>(puncture_matrix(0, p));
+  punct_total1 = punct_total2;
+  // 1st code parity bits
+  for(j = 1; j < n1 + 1; j++) {
+    for(p = 0; p < Period; p++)
+      punct_total1 += static_cast<int>(puncture_matrix(j, p));
+  }
+  // 2nd code parity bits
+  for(j = 1 + n1; j < n_tot; j++)  {
+    for(p = 0; p < Period; p++)
+      punct_total2 += static_cast<int>(puncture_matrix(j, p));
+  }
+
+  // nominal rate
+  rate = Period / static_cast<double>(punct_total);
+  calculate_punctured_size();
+}
+
+
+double Punctured_Turbo_Codec::get_rate(bool nominal)
+{
+  if(nominal) return rate;
+  else {
+    if(Period == 0)
+      return static_cast<double>(Nuncoded) / Ncoded;
+    else
+      return static_cast<double>(Nuncoded) / pNcoded;
+  }
+}
+
+
+bvec Punctured_Turbo_Codec::encode(const bvec &input)
+{
+  bvec coded_bits;
+
+  encode(input, coded_bits);
+  return coded_bits;
+}
+
+bvec Punctured_Turbo_Codec::decode(const vec &received_signal)
+{
+  bvec decoded_bits;
+
+  decode(received_signal, decoded_bits);
+  return decoded_bits;
+}
+
+void Punctured_Turbo_Codec::encode(const bvec &input, bvec &output)
+{
+  it_assert(Period != 0, "Punctured_Turbo_Codec: puncture matrix is not set");
+
+  Turbo_Codec::encode(input, output);
+
+  int i, k, p, j, p1;
+  int no_blocks = output.size() / Ncoded;
+  int count = 0, count_p = 0;
+
+  for(k = 0; k < no_blocks; k++) {
+    p = 0;
+    // data
+    for(i = 0; i < Nuncoded; i++) {
+      for(j = 0; j < n_tot; j++) {
+        if(puncture_matrix(j, p) == bin(1)) {
+          output(count_p) = output(count);
+          count_p++;
+        }
+        count++;
+      }
+      p = (p + 1) % Period;
+    }
+    p1 = p;
+
+    //The first tail:
+    for(i = 0; i < m_tail; i++) {
+      for(j = 0; j < n1 + 1; j++) {
+        if(puncture_matrix(j, p) == bin(1)) {
+          output(count_p) = output(count);
+          count_p++;
+        }
+        count++;
+      }
+      p = (p + 1) % Period;
+    }
+    //The second tail:
+    for(i = 0; i < m_tail; i++) {
+      // systematic bit
+      if(puncture_matrix(0, p1) == bin(1)) {
+        output(count_p) = output(count);
+        count_p++;
+      }
+      count++;
+      // parity
+      for(j = n1 + 1; j < n_tot; j++) {
+        if(puncture_matrix(j, p1) == bin(1)) {
+          output(count_p) = output(count);
+          count_p++;
+        }
+        count++;
+      }
+      p1 = (p1 + 1) % Period;
+    }
+  }  //for
+
+  output.set_size(count_p, true);
+}
+
+
+void Punctured_Turbo_Codec::decode(const vec &received_signal, bvec &decoded_bits, ivec &nrof_used_iterations, const bvec &true_bits)
+{
+  int i, k, p, j, p1;
+  int index = 0, index_p = 0;
+  int no_blocks = received_signal.size() / pNcoded;
+  vec temp(no_blocks * Ncoded);
+
+  it_assert(Period != 0, "Punctured_Turbo_Codec: puncture matrix is not set");
+  it_assert(no_blocks * pNcoded == received_signal.size(), "Punctured_Turbo_Codec: received vector is not an integer multiple of encoded block");
+  for(i = 0; i < no_blocks; i++)  {
+    p = 0;
+    // data
+    for(k = 0; k < Nuncoded; k++)  {
+      for(j = 0; j < n_tot; j++) {
+        if(puncture_matrix(j, p) == bin(1)) {
+          temp(index) = received_signal(index_p);
+          index_p++;
+        }
+        else { // insert dummy symbols with same contribution for 0 and 1
+          temp(index) = 0;
+        }
+        index++;
+      }
+      p = (p + 1) % Period;
+    } // for
+    p1 = p;
+
+    // 1st code tail
+    for(k = 0; k < m_tail; k++) {
+      for(j = 0; j < n1 + 1; j++) {
+        if(puncture_matrix(j, p) == bin(1)) {
+          temp(index) = received_signal(index_p);
+          index_p++;
+        }
+        else { // insert dummy symbols with same contribution for 0 and 1
+          temp(index) = 0;
+        }
+        index++;
+      }
+      p = (p + 1) % Period;
+    } // for
+    // 2nd code tail
+    for(k = 0; k < m_tail; k++) {
+      // systematic bits
+      if(puncture_matrix(0, p1) == bin(1)) {
+        temp(index) = received_signal(index_p);
+        index_p++;
+      }
+      else { // insert dummy symbols with same contribution for 0 and 1
+        temp(index) = 0;
+      }
+      index++;
+      // parity bits
+      for(j = n1 + 1; j < n_tot; j++) {
+        if(puncture_matrix(j, p1) == bin(1)) {
+          temp(index) = received_signal(index_p);
+          index_p++;
+        }
+        else { // insert dummy symbols with same contribution for 0 and 1
+          temp(index) = 0;
+        }
+        index++;
+      }
+      p1 = (p1 + 1) % Period;
+    } //2nd tail
+  }  // for
+
+  Turbo_Codec::decode(temp, decoded_bits, nrof_used_iterations, true_bits);
+}
+
+void Punctured_Turbo_Codec::decode(const vec &received_signal, bvec &decoded_bits, const bvec &true_bits)
+{
+  ivec nrof_used_iterations;
+  decode(received_signal, decoded_bits, nrof_used_iterations, true_bits);
+}
+
+void Punctured_Turbo_Codec::calculate_punctured_size(void)
+{
+  int i, j, ii, p = 0, p1;
+
+  if(Period == 0)
+    pNcoded = Ncoded;
+  else  {
+    i = (Nuncoded / Period);
+    ii = i * punct_total;
+    i *= Period;
+    for(; i < Nuncoded; i++) {
+      for(j = 0; j < n_tot; j++)
+        if(puncture_matrix(j, p) == bin(1))  ii++;
+      p = (p + 1) % Period;
+    }
+    p1 = p;
+
+    // first tail
+    for(i = 0; i < m_tail; i++) {
+      for(j = 0; j < n1 + 1; j++)
+        if(puncture_matrix(j, p) == bin(1))  ii++;
+      p = (p + 1) % Period;
+    }
+    // second tail
+    for(i = 0; i < m_tail; i++) {
+      for(j = 0; j < n_tot; j++)  {
+        if(puncture_matrix(j, p1) == bin(1))  ii++;
+        if(j == 0) j += n1;
+      }
+      p1 = (p1 + 1) % Period;
+    }
+
+    pNcoded = ii;
+  }
+}
+
+int calculate_uncoded_size(Punctured_Turbo_Codec &tc, int punctured_size, int &fill_bits)
+{
+  // fill_bits - number of bits that must be added at the end of encoded block (in order to obtain punctured_size length vector)
+
+  int Nuncoded;
+
+  if(tc.Period == 0) {
+    Nuncoded = (punctured_size - tc.m_tail * (tc.n_tot + 1)) / tc.n_tot;
+    fill_bits = punctured_size - (Nuncoded * tc.n_tot + tc.m_tail * (tc.n_tot + 1));
+  }
+  else  {
+    int i, j, ii, p, p1, no_pblocks;
+    // uncoded - // no_pblocks might be too small
+    j = static_cast<int>(std::ceil(static_cast<double>(tc.m_tail * (tc.punct_total1 + tc.punct_total2)) / tc.Period));
+    no_pblocks = (punctured_size - j) / tc.punct_total;
+    ii = punctured_size - no_pblocks * tc.punct_total - j;
+
+    for(i = 0; i < 2 * tc.Period; i++) {
+      for(j = 0; j < tc.n_tot; j++)
+        if(tc.puncture_matrix(j, i % tc.Period) == bin(1))  ii--;
+      if(ii < 0) break;
+    }
+    Nuncoded = no_pblocks * tc.Period + i;
+
+    // punctured (from uncoded)
+    no_pblocks = (Nuncoded / tc.Period);
+    ii = no_pblocks * tc.punct_total;
+    p = 0;
+    for(i = no_pblocks * tc.Period; i < Nuncoded; i++) {
+      for(j = 0; j < tc.n_tot; j++)
+        if(tc.puncture_matrix(j, p) == bin(1))  ii++;
+      p = (p + 1) % tc.Period;
+    }
+    p1 = p;
+    // first tail
+    for(i = 0; i < tc.m_tail; i++) {
+      for(j = 0; j < tc.n1 + 1; j++)
+        if(tc.puncture_matrix(j, p1) == bin(1))  ii++;
+      p1 = (p1 + 1) % tc.Period;
+    }
+    // second tail
+    for(i = 0; i < tc.m_tail; i++) {
+      for(j = 0; j < tc.n_tot; j++)  {
+        if(tc.puncture_matrix(j, p1) == bin(1))  ii++;
+        if(j == 0) j += tc.n1;
+      }
+      p1 = (p1 + 1) % tc.Period;
+    }
+    fill_bits =  punctured_size - ii;
+  }
+
+  return Nuncoded;
+}
+
+
+// -------------------------------------------------------------------------------------
+// Special interleaver sequence generators
+// -------------------------------------------------------------------------------------
+
 ivec wcdma_turbo_interleaver_sequence(int interleaver_size)
 {
   const int MAX_INTERLEAVER_SIZE = 5114;
@@ -622,7 +948,7 @@ ivec wcdma_turbo_interleaver_sequence(int interleaver_size)
   ivec primes, roots, Pat1, Pat2, Pat3, Pat4, Isort;
   int i, j, qj, temp, row, col, index, count;
 
-  if (interleaver_size > MAX_INTERLEAVER_SIZE) {
+  if(interleaver_size > MAX_INTERLEAVER_SIZE) {
 
     I = sort_index(randu(interleaver_size));
     return I;
@@ -644,10 +970,10 @@ ivec wcdma_turbo_interleaver_sequence(int interleaver_size)
     roots = "0 0 0 3 2 2 3 2 5 2 3 2 6 3 5 2 2 2 2 7 5 3 2 3 5 2 5 2 6 3 3 2 3 2 2 6 5 2 5 2 2 2 19 5 2 3 2 3 2 6 3 7 7 6 3";
 
     //Determine R
-    if ((K >= 40) && (K <= 159)) {
+    if((K >= 40) && (K <= 159)) {
       R = 5;
     }
-    else if (((K >= 160) && (K <= 200)) || ((K >= 481) && (K <= 530))) {
+    else if(((K >= 160) && (K <= 200)) || ((K >= 481) && (K <= 530))) {
       R = 10;
     }
     else {
@@ -655,23 +981,23 @@ ivec wcdma_turbo_interleaver_sequence(int interleaver_size)
     }
 
     //Determine C
-    if ((K >= 481) && (K <= 530)) {
+    if((K >= 481) && (K <= 530)) {
       p = 53;
       v = 2;
       C = p;
     }
     else {
       //Find minimum prime p such that (p+1) - K/R >= 0 ...
-      for (i = 0; i < primes.length(); i++) {
-        if ((double(primes(i) + 1) - double(K) / double(R)) >= 0.0) {
+      for(i = 0; i < primes.length(); i++) {
+        if((double(primes(i) + 1) - double(K) / double(R)) >= 0.0) {
           p = primes(i);
           v = roots(i);
           break;
         }
       }
       //... and etermine C such that
-      if ((double(p) - double(K) / double(R)) >= 0.0) {
-        if ((double(p) - 1.0 - double(K) / double(R)) >= 0.0) {
+      if((double(p) - double(K) / double(R)) >= 0.0) {
+        if((double(p) - 1.0 - double(K) / double(R)) >= 0.0) {
           C = p - 1;
         }
         else {
@@ -687,7 +1013,7 @@ ivec wcdma_turbo_interleaver_sequence(int interleaver_size)
     s.set_size(p - 1, false);
     s.clear();
     s(0) = 1;
-    for (i = 1; i <= (p - 2); i++) {
+    for(i = 1; i <= (p - 2); i++) {
       s(i) = mod(v * s(i - 1), p);
     }
 
@@ -696,11 +1022,11 @@ ivec wcdma_turbo_interleaver_sequence(int interleaver_size)
     q.set_size(R, false);
     q.clear();
     q(0) = 1;
-    for (j = 1; j <= (R - 1); j++) {
-      for (i = 0; i < primes.length(); i++) {
+    for(j = 1; j <= (R - 1); j++) {
+      for(i = 0; i < primes.length(); i++) {
         qj = primes(i);
-        if ((qj > 6) && (qj > q(j - 1))) {
-          if (gcd(qj, p - 1) == 1) {
+        if((qj > 6) && (qj > q(j - 1))) {
+          if(gcd(qj, p - 1) == 1) {
             q(j) = qj;
             break;
           }
@@ -716,28 +1042,28 @@ ivec wcdma_turbo_interleaver_sequence(int interleaver_size)
 
     //T(j) is the inter-row permutation patters defined as one of the following four
     //kinds of patterns: Pat1, Pat2, Pat3, and Pat4 depending on the number of input bits K
-    if (K >= 3211) {
+    if(K >= 3211) {
       T = Pat1;
     }
-    else if (K >= 3161) {
+    else if(K >= 3161) {
       T = Pat2;
     }
-    else if (K >= 2481) {
+    else if(K >= 2481) {
       T = Pat1;
     }
-    else if (K >= 2281) {
+    else if(K >= 2281) {
       T = Pat2;
     }
-    else if (K >= 531) {
+    else if(K >= 531) {
       T = Pat1;
     }
-    else if (K >= 481) {
+    else if(K >= 481) {
       T = Pat3;
     }
-    else if (K >= 201) {
+    else if(K >= 201) {
       T = Pat1;
     }
-    else if (K >= 160) {
+    else if(K >= 160) {
       T = Pat3;
     }
     else {
@@ -748,7 +1074,7 @@ ivec wcdma_turbo_interleaver_sequence(int interleaver_size)
     //where T(j) indicates the original row position of the j-th permuted row
     r.set_size(R, false);
     r.clear();
-    for (j = 0; j <= (R - 1); j++) {
+    for(j = 0; j <= (R - 1); j++) {
       r(T(j)) = q(j);
     }
 
@@ -756,31 +1082,31 @@ ivec wcdma_turbo_interleaver_sequence(int interleaver_size)
     //Perform the j-th (j=0, 1, 2, ..., (R-1)) intra-row permutation as
     U.set_size(R, C, false);
     U.clear();
-    if (C == p) {
-      for (j = 0; j <= (R - 1); j++) {
-        for (i = 0; i <= (p - 2); i++) {
+    if(C == p) {
+      for(j = 0; j <= (R - 1); j++) {
+        for(i = 0; i <= (p - 2); i++) {
           U(j, i) = s(mod(i * r(j), p - 1));
         }
         U(j, p - 1) = 0;
       }
     }
-    else if (C == (p + 1)) {
-      for (j = 0; j <= (R - 1); j++) {
-        for (i = 0; i <= (p - 2); i++) {
+    else if(C == (p + 1)) {
+      for(j = 0; j <= (R - 1); j++) {
+        for(i = 0; i <= (p - 2); i++) {
           U(j, i) = s(mod(i * r(j), p - 1));
         }
         U(j, p - 1) = 0;
         U(j, p) = p;
       }
-      if (K == (C*R)) {
+      if(K == (C * R)) {
         temp = U(R - 1, p);
         U(R - 1, p) = U(R - 1, 0);
         U(R - 1, 0) = temp;
       }
     }
-    else if (C == (p - 1)) {
-      for (j = 0; j <= (R - 1); j++) {
-        for (i = 0; i <= (p - 2); i++) {
+    else if(C == (p - 1)) {
+      for(j = 0; j <= (R - 1); j++) {
+        for(i = 0; i <= (p - 2); i++) {
           U(j, i) = s(mod(i * r(j), p - 1)) - 1;
         }
       }
@@ -790,12 +1116,12 @@ ivec wcdma_turbo_interleaver_sequence(int interleaver_size)
     I.set_size(K, false);
     I.clear();
     count = 0;
-    for (i = 0; i < C; i++) {
-      for (j = 0; j < R; j++) {
+    for(i = 0; i < C; i++) {
+      for(j = 0; j < R; j++) {
         row = T(j);
         col = U(row, i);
         index = row * C + col;
-        if (index < K) {
+        if(index < K) {
           I(count) = index;
           count++;
         }
@@ -806,4 +1132,59 @@ ivec wcdma_turbo_interleaver_sequence(int interleaver_size)
   }
 }
 
+ivec lte_turbo_interleaver_sequence(int interleaver_size)
+// for standard see pp. 14 http://www.3gpp.org/FTP/Specs/latest/Rel-10/36_series/36212-a50.zip
+{
+
+  //Definitions of block lengths and associated f1 and f2 factors:
+  ivec block_lengths("40 48 56 64 72 80 88 96 104 112 120 128 136 144 152 160 168 176 184 192 200 208 216 224 232 240 248 256 264 272 280 288 296 304 312 320 328 336 344 352 360 368 376 384 392 400 408 416 424 432 440 448 456 464 472 480 488 496 504 512 528 544 560 576 592 608 624 640 656 672 688 704 720 736 752 768 784 800 816 832 848 864 880 896 912 928 944 960 976 992 1008 1024 1056 1088 1120 1152 1184 1216 1248 1280 1312 1344 1376 1408 1440 1472 1504 1536 1568 1600 1632 1664 1696 1728 1760 1792 1824 1856 1888 1920 1952 1984 2016 2048 2112 2176 2240 2304 2368 2432 2496 2560 2624 2688 2752 2816 2880 2944 3008 3072 3136 3200 3264 3328 3392 3456 3520 3584 3648 3712 3776 3840 3904 3968 4032 4096 4160 4224 4288 4352 4416 4480 4544 4608 4672 4736 4800 4864 4928 4992 5056 5120 5184 5248 5312 5376 5440 5504 5568 5632 5696 5760 5824 5888 5952 6016 6080 6144");
+  ivec f1_factors(" 3  7 19  7  7 11  5 11   7  41 103  15   9  17   9  21 101  21  57  23  13  27  11  27  85  29  33  15  17  33 103  19  19  37  19  21  21 115 193  21 133  81  45  23 243 151 155  25  51  47  91  29  29 247  29  89  91 157  55  31  17  35 227  65  19  37  41  39 185  43  21 155  79 139  23 217  25  17 127  25 239  17 137 215  29  15 147  29  59  65   55   31   17  171   67   35   19   39   19  199   21  211   21   43  149   45   49   71   13   17   25  183   55  127   27   29   29   57   45   31   59  185  113   31   17  171  209  253  367  265  181   39   27  127  143   43   29   45  157   47   13  111  443   51   51  451  257   57  313  271  179  331  363  375  127   31   33   43   33  477   35  233  357  337   37   71   71   37   39  127   39   39   31  113   41  251   43   21   43   45   45  161   89  323   47   23   47  263");
+  ivec f2_factors("10 12 42 16 18 20 22 24  26  84  90  32  34 108  38 120  84  44  46  48  50  52  36  56  58  60  62  32 198  68 210  36  74  76  78 120  82  84  86  44  90  46  94  48  98  40 102  52 106  72 110 168 114  58 118 180 122  62  84  64  66  68 420  96  74  76 234  80  82 252  86  44 120  92  94  48  98  80 102  52 106  48 110 112 114  58 118  60 122 124   84   64   66  204  140   72   74   76   78  240   82  252   86   88   60   92  846   48   28   80  102  104  954   96  110  112  114  116  354  120  610  124  420   64   66  136  420  216  444  456  468   80  164  504  172   88  300   92  188   96   28  240  204  104  212  192  220  336  228  232  236  120  244  248  168   64  130  264  134  408  138  280  142  480  146  444  120  152  462  234  158   80   96  902  166  336  170   86  174  176  178  120  182  184  186   94  190  480");
+  const int MAX_INTERLEAVER_SIZE = 6144;
+  const int MIN_INTERLEAVER_SIZE = 40;
+
+  // Check the range of the interleaver size:
+  it_assert(interleaver_size <= MAX_INTERLEAVER_SIZE, "lte_turbo_interleaver_sequence: The interleaver size is too large");
+  it_assert(interleaver_size >= MIN_INTERLEAVER_SIZE, "lte_turbo_interleaver_sequence: The interleaver size is too small");
+
+  // Check whether the given interleaver size is correct:
+  int left, right, index, temp;
+  bool search = true;
+
+  // do a binary search for interleaver_size in block_lengths
+  left = 0;
+  right = block_lengths.size() - 1;
+  temp = 0;
+  while((search) && (left <= right)) {
+    index = (left + right) / 2;
+    temp = block_lengths(index);
+    if(temp == interleaver_size) {
+      search = false;
+    }
+    else {
+      if(temp > interleaver_size) {
+        right = index - 1;
+      }
+      else {
+        left = index + 1;
+      }
+    }
+  }
+  it_assert(!search, "lte_turbo_interleaver_sequence: The interleaver size is incorrect!");
+
+  // Definitions of key parameters:
+  int K = interleaver_size; // Interleaver size
+  int f1_factor = f1_factors(index);
+  int f2_factor = f2_factors(index);
+  ivec I(K); //The interleaver sequence
+
+  // Calculate the interleaver sequence:
+  for(int i = 0; i < K; i++) {
+    I(i) = static_cast<int>((static_cast<int64_t>(i) * f1_factor + static_cast<int64_t>(i) * i * f2_factor) % K);
+  }
+
+  return I;
+}
+
+
 } // namespace itpp
