@@ -32,7 +32,8 @@
 #include <itpp/base/vec.h>
 #include <itpp/base/array.h>
 #include <limits>
-
+#include <itpp/itexports.h>
+#include <itpp/base/base_exports.h>
 
 namespace itpp
 {
@@ -68,7 +69,7 @@ enum Newton_Search_Method {BFGS};
   \f$\varepsilon_1 = 10^{-4}\f$ an \f$\varepsilon_2 = 10^{-8}\f$.
 
 */
-class Newton_Search
+class ITPP_EXPORT Newton_Search
 {
 public:
   //! Default constructor
@@ -219,7 +220,7 @@ enum Line_Search_Method {Soft, Exact};
   The exact line search can at least in theory give the exact resutl, but it may require
   many extra function evaluations compared to soft line search.
 */
-class Line_Search
+class ITPP_EXPORT Line_Search
 {
 public:
   //! Default constructor
@@ -332,7 +333,7 @@ private:
 
   The default algorithm is a Quasi-Newton search using BFGS updates of the inverse Hessian matrix.
 */
-vec fminunc(double(*function)(const vec&), vec(*gradient)(const vec&), const vec &x0);
+ITPP_EXPORT vec fminunc(double(*function)(const vec&), vec(*gradient)(const vec&), const vec &x0);
 
 //@}
 
