@@ -29,9 +29,19 @@
 #ifndef SELECTIVE_REPEAT_H
 #define SELECTIVE_REPEAT_H
 
+#include <itpp/base/vec.h>
+
+#if (defined(_MSC_VER) && defined(ITPP_SHARED_LIB) && !(defined(itpp_EXPORTS) || defined(itpp_debug_EXPORTS)))
+
+#ifndef ITPP_PROTOCOL_EXCLUDED
+#define ITPP_PROTOCOL_EXCLUDED
+#pragma message( "PROTOCOL definitions are not available for MSVC shared builds" )
+#endif
+
+#else
+
 #include <itpp/protocol/packet.h>
 #include <itpp/protocol/front_drop_queue.h>
-#include <itpp/base/vec.h>
 #include <itpp/base/array.h>
 
 
@@ -167,5 +177,7 @@ private:
 
 } // namespace itpp
 
+#endif
+
 #endif // #ifndef SELECTIVE_REPEAT_H
 
