@@ -5,24 +5,23 @@
  *
  * -------------------------------------------------------------------------
  *
- * IT++ - C++ library of mathematical, signal processing, speech processing,
- *        and communications classes and functions
+ * Copyright (C) 1995-2010  (see AUTHORS file for a list of contributors)
  *
- * Copyright (C) 1995-2008  (see AUTHORS file for a list of contributors)
+ * This file is part of IT++ - a C++ library of mathematical, signal
+ * processing, speech processing, and communications classes and functions.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * IT++ is free software: you can redistribute it and/or modify it under the
+ * terms of the GNU General Public License as published by the Free Software
+ * Foundation, either version 3 of the License, or (at your option) any
+ * later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * IT++ is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ * You should have received a copy of the GNU General Public License along
+ * with IT++.  If not, see <http://www.gnu.org/licenses/>.
  *
  * -------------------------------------------------------------------------
  */
@@ -30,14 +29,8 @@
 #ifndef FILTER_H
 #define FILTER_H
 
-#ifndef _MSC_VER
-#  include <itpp/config.h>
-#else
-#  include <itpp/config_msvc.h>
-#endif
-
 #include <itpp/base/vec.h>
-
+#include <itpp/itexports.h>
 
 namespace itpp
 {
@@ -181,7 +174,7 @@ private:
 
   Vec<T3> mem;
   Vec<T2> coeffs;
-  T2 a0;
+  Vec<T2> a0;
   int inptr;
   bool init;
 };
@@ -272,36 +265,36 @@ Setting a=1 gives a MA filter and b=1 gives a AR filter. The
 
   @{
 */
-vec filter(const vec &b, const vec &a, const vec &input);
-cvec filter(const vec &b, const vec &a, const cvec &input);
-cvec filter(const cvec &b, const cvec &a, const cvec &input);
-cvec filter(const cvec &b, const cvec &a, const vec &input);
+ITPP_EXPORT vec filter(const vec &b, const vec &a, const vec &input);
+ITPP_EXPORT cvec filter(const vec &b, const vec &a, const cvec &input);
+ITPP_EXPORT cvec filter(const cvec &b, const cvec &a, const cvec &input);
+ITPP_EXPORT cvec filter(const cvec &b, const cvec &a, const vec &input);
 
-vec filter(const vec &b, const int one, const vec &input);
-cvec filter(const vec &b, const int one, const cvec &input);
-cvec filter(const cvec &b, const int one, const cvec &input);
-cvec filter(const cvec &b, const int one, const vec &input);
+ITPP_EXPORT vec filter(const vec &b, const int one, const vec &input);
+ITPP_EXPORT cvec filter(const vec &b, const int one, const cvec &input);
+ITPP_EXPORT cvec filter(const cvec &b, const int one, const cvec &input);
+ITPP_EXPORT cvec filter(const cvec &b, const int one, const vec &input);
 
-vec filter(const int one, const vec &a, const vec &input);
-cvec filter(const int one, const vec &a, const cvec &input);
-cvec filter(const int one, const cvec &a, const cvec &input);
-cvec filter(const int one, const cvec &a, const vec &input);
+ITPP_EXPORT vec filter(const int one, const vec &a, const vec &input);
+ITPP_EXPORT cvec filter(const int one, const vec &a, const cvec &input);
+ITPP_EXPORT cvec filter(const int one, const cvec &a, const cvec &input);
+ITPP_EXPORT cvec filter(const int one, const cvec &a, const vec &input);
 
 
-vec filter(const vec &b, const vec &a, const vec &input, const vec &state_in, vec &state_out);
-cvec filter(const vec &b, const vec &a, const cvec &input, const cvec &state_in, cvec &state_out);
-cvec filter(const cvec &b, const cvec &a, const cvec &input, const cvec &state_in, cvec &state_out);
-cvec filter(const cvec &b, const cvec &a, const vec &input, const cvec &state_in, cvec &state_out);
+ITPP_EXPORT vec filter(const vec &b, const vec &a, const vec &input, const vec &state_in, vec &state_out);
+ITPP_EXPORT cvec filter(const vec &b, const vec &a, const cvec &input, const cvec &state_in, cvec &state_out);
+ITPP_EXPORT cvec filter(const cvec &b, const cvec &a, const cvec &input, const cvec &state_in, cvec &state_out);
+ITPP_EXPORT cvec filter(const cvec &b, const cvec &a, const vec &input, const cvec &state_in, cvec &state_out);
 
-vec filter(const vec &b, const int one, const vec &input, const vec &state_in, vec &state_out);
-cvec filter(const vec &b, const int one, const cvec &input, const cvec &state_in, cvec &state_out);
-cvec filter(const cvec &b, const int one, const cvec &input, const cvec &state_in, cvec &state_out);
-cvec filter(const cvec &b, const int one, const vec &input, const cvec &state_in, cvec &state_out);
+ITPP_EXPORT vec filter(const vec &b, const int one, const vec &input, const vec &state_in, vec &state_out);
+ITPP_EXPORT cvec filter(const vec &b, const int one, const cvec &input, const cvec &state_in, cvec &state_out);
+ITPP_EXPORT cvec filter(const cvec &b, const int one, const cvec &input, const cvec &state_in, cvec &state_out);
+ITPP_EXPORT cvec filter(const cvec &b, const int one, const vec &input, const cvec &state_in, cvec &state_out);
 
-vec filter(const int one, const vec &a, const vec &input, const vec &state_in, vec &state_out);
-cvec filter(const int one, const vec &a, const cvec &input, const cvec &state_in, cvec &state_out);
-cvec filter(const int one, const cvec &a, const cvec &input, const cvec &state_in, cvec &state_out);
-cvec filter(const int one, const cvec &a, const vec &input, const cvec &state_in, cvec &state_out);
+ITPP_EXPORT vec filter(const int one, const vec &a, const vec &input, const vec &state_in, vec &state_out);
+ITPP_EXPORT cvec filter(const int one, const vec &a, const cvec &input, const cvec &state_in, cvec &state_out);
+ITPP_EXPORT cvec filter(const int one, const cvec &a, const cvec &input, const cvec &state_in, cvec &state_out);
+ITPP_EXPORT cvec filter(const int one, const cvec &a, const vec &input, const cvec &state_in, cvec &state_out);
 /*! @} */
 
 
@@ -310,7 +303,7 @@ cvec filter(const int one, const cvec &a, const vec &input, const cvec &state_in
   cutoff using the window method.
   \ingroup filters
 */
-vec fir1(int N, double cutoff);
+ITPP_EXPORT vec fir1(int N, double cutoff);
 
 //----------------------------------------------------------------------------
 // Implementation of templated functions starts here
@@ -428,8 +421,9 @@ void AR_Filter<T1, T2, T3>::set_coeffs(const Vec<T2> &a)
   it_assert(a.size() > 0, "AR_Filter: size of filter is 0!");
   it_assert(a(0) != T2(0), "AR_Filter: a(0) cannot be 0!");
 
-  a0 = a(0);
-  coeffs = a / a0;
+  a0.set_size(1);//needed to keep the first coefficient for future reuse
+  a0(0) = a(0);
+  coeffs = a / a0(0);
 
   mem.set_size(coeffs.size() - 1, false);
   mem.clear();
@@ -471,7 +465,7 @@ T3 AR_Filter<T1, T2, T3>::filter(const T1 Sample)
   T3 s = Sample;
 
   if (mem.size() == 0)
-    return (s / a0);
+    return (s / a0(0));
 
   int L = mem.size() - inptr;
   for (int i = 0; i < L; i++) {
@@ -486,7 +480,7 @@ T3 AR_Filter<T1, T2, T3>::filter(const T1 Sample)
     inptr += mem.size();
   mem(inptr) = s;
 
-  return (s / a0);
+  return (s / a0(0));
 }
 
 
@@ -576,35 +570,30 @@ T3 ARMA_Filter<T1, T2, T3>::filter(const T1 Sample)
 // ----------------------------------------------------------------------
 // Instantiations
 // ----------------------------------------------------------------------
-
-#ifdef HAVE_EXTERN_TEMPLATE
-
-extern template class MA_Filter<double, double, double>;
-extern template class MA_Filter < double, std::complex<double>,
+ITPP_EXPORT_TEMPLATE template class ITPP_EXPORT MA_Filter<double, double, double>;
+ITPP_EXPORT_TEMPLATE template class ITPP_EXPORT MA_Filter< double, std::complex<double>,
   std::complex<double> >;
-extern template class MA_Filter < std::complex<double>, double,
+ITPP_EXPORT_TEMPLATE template class ITPP_EXPORT MA_Filter< std::complex<double>, double,
   std::complex<double> >;
-extern template class MA_Filter < std::complex<double>, std::complex<double>,
+ITPP_EXPORT_TEMPLATE template class ITPP_EXPORT MA_Filter< std::complex<double>, std::complex<double>,
   std::complex<double> >;
 
-extern template class AR_Filter<double, double, double>;
-extern template class AR_Filter < double, std::complex<double>,
+ITPP_EXPORT_TEMPLATE template class ITPP_EXPORT AR_Filter<double, double, double>;
+ITPP_EXPORT_TEMPLATE template class ITPP_EXPORT AR_Filter< double, std::complex<double>,
   std::complex<double> >;
-extern template class AR_Filter < std::complex<double>,
+ITPP_EXPORT_TEMPLATE template class ITPP_EXPORT AR_Filter< std::complex<double>,
   double, std::complex<double> >;
-extern template class AR_Filter < std::complex<double>, std::complex<double>,
+ITPP_EXPORT_TEMPLATE template class ITPP_EXPORT AR_Filter< std::complex<double>, std::complex<double>,
   std::complex<double> >;
 
-extern template class ARMA_Filter<double, double, double>;
-extern template class ARMA_Filter < double, std::complex<double>,
+ITPP_EXPORT_TEMPLATE template class ITPP_EXPORT ARMA_Filter<double, double, double>;
+ITPP_EXPORT_TEMPLATE template class ITPP_EXPORT ARMA_Filter< double, std::complex<double>,
   std::complex<double> >;
-extern template class ARMA_Filter < std::complex<double>,
+ITPP_EXPORT_TEMPLATE template class ITPP_EXPORT ARMA_Filter< std::complex<double>,
   double, std::complex<double> >;
-extern template class ARMA_Filter < std::complex<double>, std::complex<double>,
+ITPP_EXPORT_TEMPLATE template class ITPP_EXPORT ARMA_Filter< std::complex<double>, std::complex<double>,
   std::complex<double> >;
 
-#endif // HAVE_EXTERN_TEMPLATE
-
 //! \endcond
 
 } // namespace itpp
