@@ -5,24 +5,23 @@
  *
  * -------------------------------------------------------------------------
  *
- * IT++ - C++ library of mathematical, signal processing, speech processing,
- *        and communications classes and functions
+ * Copyright (C) 1995-2010  (see AUTHORS file for a list of contributors)
  *
- * Copyright (C) 1995-2009  (see AUTHORS file for a list of contributors)
+ * This file is part of IT++ - a C++ library of mathematical, signal
+ * processing, speech processing, and communications classes and functions.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * IT++ is free software: you can redistribute it and/or modify it under the
+ * terms of the GNU General Public License as published by the Free Software
+ * Foundation, either version 3 of the License, or (at your option) any
+ * later version.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * IT++ is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ * You should have received a copy of the GNU General Public License along
+ * with IT++.  If not, see <http://www.gnu.org/licenses/>.
  *
  * -------------------------------------------------------------------------
  */
@@ -34,6 +33,7 @@
 #include <itpp/base/math/elem_math.h>
 #include <itpp/base/matfunc.h>
 #include <itpp/base/specmat.h>
+#include <itpp/base/itcompat.h>
 #include <itpp/stat/misc_stat.h>
 
 
@@ -92,7 +92,7 @@ void xcorr(const vec &x, const vec &y, vec &out, const int max_lag, const std::s
 void xcorr_old(const vec &x, const vec &y, vec &out, const int max_lag, const std::string scaleopt)
 {
   int m, n;
-  double s_plus, s_minus, M_double, xcorr_0, coeff_scale = 0.0;
+  double s_plus, s_minus, M_double, coeff_scale = 0.0;
   int M, N;
 
   M = x.size();
@@ -123,8 +123,6 @@ void xcorr_old(const vec &x, const vec &y, vec &out, const int max_lag, const st
       s_plus += index_zero_pad(x, n + m) * index_zero_pad(y, n);
     }
 
-    if (m == 0) { xcorr_0 = s_plus; }
-
     if (scaleopt == "none") {
       out(N + m - 1) = s_plus;
       out(N - m - 1) = s_minus;
