@@ -2,11 +2,11 @@
  * \file
  * \brief Fourier, Hadamard, Walsh-Hadamard, and 2D Hadamard transforms -
  *        header file
- * \author Tony Ottosson, Thomas Eriksson, Simon Wood and Adam Piatyszek
+ * \author Tony Ottosson, Thomas Eriksson, Simon Wood, Adam Piatyszek, Andy Panov and Bogdan Cristea
  *
  * -------------------------------------------------------------------------
  *
- * Copyright (C) 1995-2010  (see AUTHORS file for a list of contributors)
+ * Copyright (C) 1995-2013  (see AUTHORS file for a list of contributors)
  *
  * This file is part of IT++ - a C++ library of mathematical, signal
  * processing, speech processing, and communications classes and functions.
@@ -33,6 +33,7 @@
 #include <itpp/base/vec.h>
 #include <itpp/base/mat.h>
 #include <itpp/base/matfunc.h>
+#include <itpp/itexports.h>
 
 
 namespace itpp
@@ -81,31 +82,66 @@ namespace itpp
 //!\addtogroup fft
 //!@{
 
+//! Run-time test if library is built with Fast Fourier Transforms enabled
+ITPP_EXPORT bool have_fourier_transforms();
 //! Fast Fourier Transform
-void fft(const cvec &in, cvec &out);
+ITPP_EXPORT void fft(const cvec &in, cvec &out);
 //! Fast Fourier Transform
-cvec fft(const cvec &in);
-//! Fast Fourier Transform, with zero-padding up to size N
-cvec fft(const cvec &in, const int N);
+ITPP_EXPORT cvec fft(const cvec &in);
+/*!
+\brief Fast Fourier Transform with zero-padding up to size N
+
+First N points of input vector are used to perform the transform if N < length(in). Padding with 0's is
+performed if N > length(in).
+*/
+ITPP_EXPORT cvec fft(const cvec &in, const int N);
 //! Inverse Fast Fourier Transform
-void ifft(const cvec &in, cvec &out);
+ITPP_EXPORT void ifft(const cvec &in, cvec &out);
 //! Inverse Fast Fourier Transform
-cvec ifft(const cvec &in);
-//! Inverse Fast Fourier Transform, with zero-padding up to size N
-cvec ifft(const cvec &in, const int N);
+ITPP_EXPORT cvec ifft(const cvec &in);
+/*!
+\brief Inverse Fast Fourier Transform with zero-padding up to size N
+
+First N points of input vector are used to perform the transform if N < length(in). Padding with 0's is
+performed if N > length(in).
+*/
+ITPP_EXPORT cvec ifft(const cvec &in, const int N);
 
 //! Real Fast Fourier Transform
-void fft_real(const vec& in, cvec &out);
+ITPP_EXPORT void fft_real(const vec& in, cvec &out);
 //! Real Fast Fourier Transform
-cvec fft_real(const vec& in);
-//! Real Fast Fourier Transform, with zero-padding up to size N
-cvec fft_real(const vec &in, const int N);
-//! Inverse Real Fast Fourier Transform. Assumes even size.
-void ifft_real(const cvec &in, vec &out);
-//! Inverse Real Fast Fourier Transform. Assumes even size.
-vec ifft_real(const cvec &in);
-//! Inverse Real Fast Fourier Transform, with zero-padding up to size N
-vec ifft_real(const cvec &in, const int N);
+ITPP_EXPORT cvec fft_real(const vec& in);
+/*!
+\brief Real Fast Fourier Transform with zero-padding up to size N
+
+First N points of input vector are used to perform the transform if N < length(in). Padding with 0's is
+performed if N > length(in).
+*/
+ITPP_EXPORT cvec fft_real(const vec &in, const int N);
+/*!
+\brief Inverse Real Fast Fourier Transform.
+
+Underlying implementation assumes Hermitian symmetry of the input spectra. Results are
+unpredictable and depending on the implementation (MKL/ACML/FFTW) if this requirement is not met.
+*/
+ITPP_EXPORT void ifft_real(const cvec &in, vec &out);
+/*!
+\brief Inverse Real Fast Fourier Transform.
+
+Underlying implementation assumes Hermittian symmetry of the input spectra. Results are
+unpredictable and depending on the implementation (MKL/ACML/FFTW) if this requirement is not met.
+*/
+ITPP_EXPORT vec ifft_real(const cvec &in);
+/*!
+\brief Inverse Real Fast Fourier Transformon with zero-padding up to size N.
+
+First N points of input vector are used to perform the transform if N < length(in). Padding with 0's is
+performed if N > length(in).
+
+Underlying implementation assumes Hermitian symmetry of the input subvector/padded sequence. Results are
+unpredictable and depending on the implementation (MKL/ACML/FFTW) if this requirement is not  met.
+*/
+ITPP_EXPORT vec ifft_real(const cvec &in, const int N);
 //!@}
 
 
@@ -129,8 +165,8 @@ vec ifft_real(const cvec &in, const int N);
 
   The implementation is built upon one of the following libraries:
   - FFTW (version 3.0.0 or higher)
-  - MKL (version 8.0.0 or higher)
-  - ACML (version 2.5.3 or higher).
+  - MKL (version 10.0.0 or higher)
+  - ACML (version 4.4.0 or higher).
 
   \note FFTW-based implementation is the fastest for powers of two.
   Furthermore, the second time you call the routine with the same size,
@@ -153,14 +189,30 @@ vec ifft_real(const cvec &in, const int N);
 //!\addtogroup dct
 //!@{
 
+//! Run-time test if library is built with cosine transforms enabled
+ITPP_EXPORT bool have_cosine_transforms();
 //! Discrete Cosine Transform (DCT)
-void dct(const vec &in, vec &out);
+ITPP_EXPORT void dct(const vec &in, vec &out);
 //! Discrete Cosine Transform (DCT)
-vec dct(const vec &in);
+ITPP_EXPORT vec dct(const vec &in);
+/*!
+\brief Discrete Cosine Transform (DCT) with zero-padding up to size N
+
+First N points of input vector are used to perform the transform if N < length(in). Padding with 0's is
+performed if N > length(in).
+*/
+ITPP_EXPORT vec dct(const vec &in, const int N);
 //! Inverse Discrete Cosine Transform (IDCT)
-void idct(const vec &in, vec &out);
+ITPP_EXPORT void idct(const vec &in, vec &out);
 //! Inverse Discrete Cosine Transform (IDCT)
-vec idct(const vec &in);
+ITPP_EXPORT vec idct(const vec &in);
+/*!
+\brief Inverse Discrete Cosine Transform (IDCT) with zero-padding up to size N
+
+First N points of input vector are used to perform the transform if N < length(in). Padding with 0's is
+performed if N > length(in).
+*/
+ITPP_EXPORT vec idct(const vec &in, const int N);
 //!@}
 
 
@@ -202,14 +254,14 @@ void bitrv(Vec<T> &out)
   int N = out.size();
   int j = 0;
   int N1 = N - 1;
-  for (int i = 0; i < N1; ++i) {
-    if (i < j) {
+  for(int i = 0; i < N1; ++i) {
+    if(i < j) {
       T temp = out[j];
       out[j] = out[i];
       out[i] = temp;
     }
     int K = N / 2;
-    while (K <= j) {
+    while(K <= j) {
       j -= K;
       K /= 2;
     }
@@ -227,18 +279,18 @@ void dht(const Vec<T> &vin, Vec<T> &vout)
   vout.set_size(N);
 
   // This step is separated because it copies vin to vout
-  for (int ib = 0; ib < N; ib += 2) {
+  for(int ib = 0; ib < N; ib += 2) {
     vout(ib) = vin(ib) + vin(ib + 1);
     vout(ib + 1) = vin(ib) - vin(ib + 1);
   }
   N /= 2;
 
   int l = 2;
-  for (int i = 1; i < m; ++i) {
+  for(int i = 1; i < m; ++i) {
     N /= 2;
     int ib = 0;
-    for (int k = 0; k < N; ++k) {
-      for (int j = 0; j < l; ++j) {
+    for(int k = 0; k < N; ++k) {
+      for(int j = 0; j < l; ++j) {
         T t = vout(ib + j);
         vout(ib + j) += vout(ib + j + l);
         vout(ib + j + l) = t - vout(ib + j + l);
@@ -260,11 +312,11 @@ void self_dht(Vec<T> &v)
                   "of two");
 
   int l = 1;
-  for (int i = 0; i < m; ++i) {
+  for(int i = 0; i < m; ++i) {
     N /= 2;
     int ib = 0;
-    for (int k = 0; k < N; ++k) {
-      for (int j = 0; j < l; ++j) {
+    for(int k = 0; k < N; ++k) {
+      for(int j = 0; j < l; ++j) {
         T t = v(ib + j);
         v(ib + j) += v(ib + j + l);
         v(ib + j + l) = t - v(ib + j + l);
@@ -306,12 +358,12 @@ Mat<T> dht2(const Mat<T> &m)
   Mat<T> ret(m.rows(), m.cols());
   Vec<T> v;
 
-  for (int i = 0; i < m.rows(); ++i) {
+  for(int i = 0; i < m.rows(); ++i) {
     v = m.get_row(i);
     self_dht(v);
     ret.set_row(i, v);
   }
-  for (int i = 0; i < m.cols(); ++i) {
+  for(int i = 0; i < m.cols(); ++i) {
     v = ret.get_col(i);
     self_dht(v);
     ret.set_col(i, v);
@@ -326,12 +378,12 @@ Mat<T> dwht2(const Mat<T> &m)
   Mat<T> ret(m.rows(), m.cols());
   Vec<T> v;
 
-  for (int i = 0; i < m.rows(); ++i) {
+  for(int i = 0; i < m.rows(); ++i) {
     v = m.get_row(i);
     self_dwht(v);
     ret.set_row(i, v);
   }
-  for (int i = 0; i < m.cols(); ++i) {
+  for(int i = 0; i < m.cols(); ++i) {
     v = ret.get_col(i);
     self_dwht(v);
     ret.set_col(i, v);
@@ -346,31 +398,27 @@ Mat<T> dwht2(const Mat<T> &m)
 // Instantiations
 // ----------------------------------------------------------------------
 
-#ifndef _MSC_VER
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT vec dht(const vec &v);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT cvec dht(const cvec &v);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT void dht(const vec &vin, vec &vout);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT void dht(const cvec &vin, cvec &vout);
 
-extern template vec dht(const vec &v);
-extern template cvec dht(const cvec &v);
-extern template void dht(const vec &vin, vec &vout);
-extern template void dht(const cvec &vin, cvec &vout);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT void self_dht(vec &v);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT void self_dht(cvec &v);
 
-extern template void self_dht(vec &v);
-extern template void self_dht(cvec &v);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT vec dwht(const vec &v);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT cvec dwht(const cvec &v);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT void dwht(const vec &vin, vec &vout);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT void dwht(const cvec &vin, cvec &vout);
 
-extern template vec dwht(const vec &v);
-extern template cvec dwht(const cvec &v);
-extern template void dwht(const vec &vin, vec &vout);
-extern template void dwht(const cvec &vin, cvec &vout);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT void self_dwht(vec &v);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT void self_dwht(cvec &v);
 
-extern template void self_dwht(vec &v);
-extern template void self_dwht(cvec &v);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT mat  dht2(const mat &m);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT cmat dht2(const cmat &m);
 
-extern template mat  dht2(const mat &m);
-extern template cmat dht2(const cmat &m);
-
-extern template mat  dwht2(const mat &m);
-extern template cmat dwht2(const cmat &m);
-
-#endif // _MSC_VER
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT mat  dwht2(const mat &m);
+ITPP_EXPORT_TEMPLATE template ITPP_EXPORT cmat dwht2(const cmat &m);
 
 //! \endcond
 
