@@ -30,7 +30,8 @@
 #define MOG_DIAG_EM_H
 
 #include <itpp/stat/mog_diag.h>
-
+#include <itpp/itexports.h>
+#include <itpp/base/base_exports.h>
 
 namespace itpp
 {
@@ -39,7 +40,7 @@ namespace itpp
   \brief support class for MOG_diag_ML() and MOG_diag_MAP()
   \author Conrad Sanderson
 */
-class MOG_diag_EM_sup : public MOG_diag
+class ITPP_EXPORT MOG_diag_EM_sup : public MOG_diag
 {
 
 public:
